@@ -43,6 +43,7 @@ extern int warn_ignore(struct soap*, const char*);
 
 extern const char *qname_token(const char*, const char*);
 extern int is_builtin_qname(const char*);
+extern xsd__QName make_qname(xs__schema&, const char *);
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -57,7 +58,7 @@ xs__schema::xs__schema()
   soap_register_plugin(soap, http_da);
 #endif
 #ifdef WITH_OPENSSL
-  soap_ssl_client_context(soap, SOAP_SSL_NO_AUTHENTICATION, NULL, NULL, NULL, NULL, NULL);
+  soap_ssl_client_context(soap, SOAP_SSL_NO_AUTHENTICATION | SOAP_SSLv3_TLSv1, NULL, NULL, NULL, NULL, NULL);
 #endif
   soap_set_namespaces(soap, namespaces);
   soap_default(soap);
@@ -126,19 +127,19 @@ int xs__schema::preprocess()
     (*im).preprocess(*this); // read schema and recurse over <import>
   for (vector<xs__include>::iterator in = include.begin(); in != include.end(); ++in)
   {
-    (*in).preprocess(*this); // read schema and recurse over <include>
+    (*in).preprocess(*this); // read schema and recurse over <include>, <override> and <redefine>
     if ((*in).schemaPtr())
       insert(*(*in).schemaPtr());
   }
   for (vector<xs__override>::iterator ov = override_.begin(); ov != override_.end(); ++ov)
   {
-    (*ov).preprocess(*this); // read schema and recurse over <override>
+    (*ov).preprocess(*this); // read schema and recurse over <include>, <override> and <redefine>
     if ((*ov).schemaPtr())
       insert(*(*ov).schemaPtr());
   }
   for (vector<xs__redefine>::iterator re = redefine.begin(); re != redefine.end(); ++re)
   {
-    (*re).preprocess(*this); // read schema and recurse over <redefine>
+    (*re).preprocess(*this); // read schema and recurse over <redefine>, <override> and <redefine>
     if ((*re).schemaPtr())
       insert(*(*re).schemaPtr());
   }
@@ -243,7 +244,7 @@ int xs__schema::insert(xs__schema& schema)
       element.back().schemaPtr(this);
     }
   }
-  // insert groups, but only add groups with new name (no conflict check)
+  // insert groups, but only add groups with new name (no conflict warning)
   for (vector<xs__group>::const_iterator gp = schema.group.begin(); gp != schema.group.end(); ++gp)
   {
     found = false;
@@ -264,7 +265,7 @@ int xs__schema::insert(xs__schema& schema)
       group.back().schemaPtr(this);
     }
   }
-  // insert attributeGroups, but only add attributeGroups with new name (no conflict check)
+  // insert attributeGroups, but only add attributeGroups with new name (no conflict warning)
   for (vector<xs__attributeGroup>::const_iterator ag = schema.attributeGroup.begin(); ag != schema.attributeGroup.end(); ++ag)
   {
     found = false;
@@ -285,7 +286,7 @@ int xs__schema::insert(xs__schema& schema)
       attributeGroup.back().schemaPtr(this);
     }
   }
-  // insert simpleTypes, but only add simpleTypes with new name (no conflict check)
+  // insert simpleTypes, but only add simpleTypes with new name (no conflict warning)
   for (vector<xs__simpleType>::const_iterator st = schema.simpleType.begin(); st != schema.simpleType.end(); ++st)
   {
     found = false;
@@ -306,7 +307,7 @@ int xs__schema::insert(xs__schema& schema)
       simpleType.back().schemaPtr(this);
     }
   }
-  // insert complexTypes, but only add complexTypes with new name (no conflict check)
+  // insert complexTypes, but only add complexTypes with new name (no conflict warning)
   for (vector<xs__complexType>::const_iterator ct = schema.complexType.begin(); ct != schema.complexType.end(); ++ct)
   {
     found = false;
@@ -408,7 +409,7 @@ int xs__schema::read(const char *cwd, const char *loc)
   if (!cwd)
     cwd = cwd_path;
   if (vflag)
-    fprintf(stderr, "\nOpening schema '%s' from '%s'\n", loc ? loc : "(stdin)", cwd ? cwd : "./");
+    fprintf(stderr, "\nOpening schema '%s' relative to '%s'\n", loc ? loc : "(stdin)", cwd ? cwd : "./");
   if (loc)
   {
     if (soap->recvfd > 2)
@@ -427,7 +428,11 @@ int xs__schema::read(const char *cwd, const char *loc)
 #else
     if (!strncmp(loc, "https://", 8))
     {
+#ifdef WIN32
+      fprintf(stderr, "\nCannot connect to https site: SSL/TLS support not enabled in this version. Visit https://www.genivia.com/downloads.html to download the secure version of wsdl2h.exe that supports SSL/TLS to connect to https sites.\n");
+#else
       fprintf(stderr, "\nCannot connect to https site: SSL/TLS support not enabled, please rebuild wsdl2h with SSL/TLS enabled using 'make secure' or download the WSDL/WADL and XSD files and rerun wsdl2h on these files directly by specifying the file names on the command line.\n");
+#endif
       exit(1);
     }
     else if (!strncmp(loc, "http://", 7))
@@ -438,6 +443,7 @@ int xs__schema::read(const char *cwd, const char *loc)
       if (soap_connect_command(soap, SOAP_GET, location, NULL))
       {
         fprintf(stderr, "\n\nError: connection failed\n");
+        soap_print_fault(soap, stderr);
         exit(1);
       }
       fprintf(stderr, " connected, receiving...\n");
@@ -458,7 +464,8 @@ int xs__schema::read(const char *cwd, const char *loc)
       fprintf(stderr, "%*sConnecting to '%s' to retrieve schema '%s'...", 2*openfiles, "", location, loc);
       if (soap_connect_command(soap, SOAP_GET, location, NULL))
       {
-        fprintf(stderr, "\n\nError: connection failed\n");
+        fprintf(stderr, "\n\nError: failed to retrieve '%s'\n", loc);
+        soap_print_fault(soap, stderr);
         exit(1);
       }
       fprintf(stderr, " connected, receiving...\n");
@@ -466,6 +473,8 @@ int xs__schema::read(const char *cwd, const char *loc)
     }
     else
     {
+      if (!strncmp(loc, "file://", 7))
+        loc += 7;
       soap->recvfd = open(loc, O_RDONLY, 0);
       if (soap->recvfd < 0)
       {
@@ -480,14 +489,18 @@ int xs__schema::read(const char *cwd, const char *loc)
             s = strrchr(location, '\\');
 #endif
           if (s)
+          {
             *s = '\0';
-          size_t n = strlen(location);
-          soap_strcpy(location + n, l + 2 - n, "/");
-          ++n;
-          soap_strcpy(location + n, l + 2 - n, loc);
-          if (!strncmp(location, "file://", 7))
-            location += 7;
-          soap->recvfd = open(location, O_RDONLY, 0);
+            size_t n = strlen(location);
+            soap_strcpy(location + n, l + 2 - n, "/");
+            ++n;
+            soap_strcpy(location + n, l + 2 - n, loc);
+            if (!strncmp(location, "file://", 7))
+              location += 7;
+            soap->recvfd = open(location, O_RDONLY, 0);
+            if (vflag)
+              cerr << "Opening file " << location << (soap->recvfd < 0 ? " failed" : " successful") << endl;
+          }
         }
         if (soap->recvfd < 0 && import_path)
         {
@@ -501,6 +514,8 @@ int xs__schema::read(const char *cwd, const char *loc)
           if (!strncmp(location, "file://", 7))
             location += 7;
           soap->recvfd = open(location, O_RDONLY, 0);
+          if (vflag)
+            cerr << "Opening file " << location << (soap->recvfd < 0 ? " failed" : " successful") << endl;
         }
         if (soap->recvfd < 0)
         {
@@ -551,7 +566,9 @@ int xs__schema::read(const char *cwd, const char *loc)
       redirs--;
     }
     else
+    {
       fprintf(stderr, "Authentication failed, use option -r:uid:pwd and (re)build with OpenSSL to enable digest authentication\n");
+    }
     return r;
   }
   if (soap->error)
@@ -572,7 +589,9 @@ int xs__schema::read(const char *cwd, const char *loc)
     soap->recvfd = -1;
   }
   else
+  {
     soap_closesock(soap);
+  }
   cwd_path = cwd_temp;
   return SOAP_OK;
 }
@@ -591,31 +610,27 @@ char *xs__schema::absoluteLocation(const char *loc) const
 {
   if (!location)
     return soap_strdup(soap, loc);
-  if (!strncmp(loc, "http://", 7) || !strncmp(loc, "https://", 8) || !strncmp(loc, "file://", 7))
+  if (!strncmp(loc, "http://", 7) || !strncmp(loc, "https://", 8))
     return soap_strdup(soap, loc);
+  if (!strncmp(loc, "file://", 7))
+    loc += 7;
   const char *s = strrchr(location, '/');
   if (!s)
     return soap_strdup(soap, loc);
-  size_t k = 0;
-  while (!strncmp(loc, "../", 3))
+  if (strncmp(loc, "../", 3))
+    return soap_strdup(soap, loc);
+  while (!strncmp(loc, "../", 3) && s > location)
   {
+    while (--s >= location)
+      if (*s == '/')
+        break;
     loc += 3;
-    ++k;
   }
-  while (k > 0 && s > location)
-    if (*--s == '/')
-      --k;
-  loc -= 3*k;
-  size_t n = s - location;
+  size_t n = s - location + 1;
   size_t l = n + strlen(loc);
-  char *abs = (char*)soap_malloc(soap, l + 2);
-  soap_strncpy(abs, l + 2, location, n);
-  if (n > 0)
-  {
-    soap_strcpy(abs + n, l + 2 - n, "/");
-    ++n;
-  }
-  soap_strcpy(abs + n, l + 2 - n, loc);
+  char *abs = (char*)soap_malloc(soap, l + 1);
+  soap_strncpy(abs, l + 1, location, n);
+  soap_strcpy(abs + n, l + 1 - n, loc);
   return abs;
 }
 
@@ -641,6 +656,11 @@ void xs__schema::builtinType(const char *type)
   builtinTypeSet.insert(type);
 }
 
+void xs__schema::builtinTypeDerivation(xs__schema& schema, const char *base, const char *derived)
+{
+  builtinTypeMap[make_qname(schema, derived)] = base;
+}
+
 void xs__schema::builtinElement(const char *element)
 {
   builtinElementSet.insert(element);
@@ -656,6 +676,11 @@ const SetOfString& xs__schema::builtinTypes() const
   return builtinTypeSet;
 }
 
+const MapOfStringToString& xs__schema::builtinTypeDerivations() const
+{
+  return builtinTypeMap;
+}
+
 const SetOfString& xs__schema::builtinElements() const
 {
   return builtinElementSet;
@@ -678,12 +703,14 @@ void xs__schema::mark()
     used = true;
     for (vector<xs__import>::iterator im = import.begin(); im != import.end(); ++im)
       (*im).mark();
-    // start with root of usage: use top-level attributes
-    for (vector<xs__attribute>::iterator at = attribute.begin(); at != attribute.end(); ++at)
-      (*at).mark();
-    // start with root of usage: use top-level elements
-    for (vector<xs__element>::iterator el = element.begin(); el != element.end(); ++el)
-      (*el).mark();
+    // -O2: start with root of usage: use top-level attributes
+    if (Oflag < 3)
+      for (vector<xs__attribute>::iterator at = attribute.begin(); at != attribute.end(); ++at)
+        (*at).mark();
+    // -O2 and -O3: start with root of usage: use top-level elements
+    if (Oflag < 4)
+      for (vector<xs__element>::iterator el = element.begin(); el != element.end(); ++el)
+        (*el).mark();
   }
 }
 
@@ -708,8 +735,8 @@ int xs__include::preprocess(xs__schema &schema)
         for (i = included.begin(); i != included.end(); ++i)
         {
           if ((*i).second->targetNamespace
-              && !strcmp(schemaLocation, (*i).first)
-              && !strcmp(schema.targetNamespace, (*i).second->targetNamespace))
+           && !strcmp(schemaLocation, (*i).first)
+           && !strcmp(schema.targetNamespace, (*i).second->targetNamespace))
             break;
         }
       }
@@ -727,13 +754,10 @@ int xs__include::preprocess(xs__schema &schema)
           if (!Wflag)
           {
             if (schemaRef->targetNamespace)
-              fprintf(stderr, "\nWarning: attempt to include schema with mismatching targetNamespace '%s' in schema '%s', assigning targetNamespace '%s'\n", schemaRef->targetNamespace, schema.targetNamespace, schema.targetNamespace);
+              fprintf(stderr, "\nWarning: attempt to include schema with mismatching targetNamespace '%s' into schema namespace '%s', assigning targetNamespace '%s'\n", schemaRef->targetNamespace, schema.targetNamespace, schema.targetNamespace);
             else
-              fprintf(stderr, "\nWarning: attempt to include chameleon schema with no targetNamespace in schema '%s', assigning targetNamespace '%s'\n", schema.targetNamespace, schema.targetNamespace);
+              fprintf(stderr, "\nWarning: attempt to include chameleon schema with no targetNamespace into schema namespace '%s', assigning targetNamespace '%s'\n", schema.targetNamespace, schema.targetNamespace);
           }
-          schemaRef->targetNamespace = schema.targetNamespace;
-          schemaRef->elementFormDefault = schema.elementFormDefault;
-          schemaRef->attributeFormDefault = schema.attributeFormDefault;
         }
       }
       else
@@ -792,11 +816,67 @@ int xs__redefine::preprocess(xs__schema &schema)
             if ((*g).name && !strcmp((*gp).name, (*g).name))
             {
               if ((*g).all)
+              {
                 (*gp).all = (*g).all;
-              if ((*g).choice)
-                (*gp).choice = (*g).choice;
-              if ((*g).sequence)
-                (*gp).sequence = (*g).sequence;
+              }
+              else if ((*g).choice && (*gp).choice)
+              {
+                xs__seqchoice& s = *(*gp).choice;
+                (*gp).choice = soap_new_xs__seqchoice(schema.soap);
+                for (vector<xs__contents>::iterator c = (*g).choice->__contents.begin(); c != (*g).choice->__contents.end(); )
+                {
+                  if ((*c).__union == SOAP_UNION_xs__union_content_element)
+                  {
+                    (*gp).choice->__contents.push_back(*c);
+                    ++c;
+                  }
+                  else if ((*c).__union == SOAP_UNION_xs__union_content_group)
+                  {
+                    if ((*c).__content.group->ref)
+                    {
+                      const char *token = qname_token((*c).__content.group->ref, schema.targetNamespace);
+                      if (token && !strcmp((*gp).name, token))
+                        for (vector<xs__contents>::const_iterator d = s.__contents.begin(); d != s.__contents.end(); ++d)
+                          if ((*d).__union == SOAP_UNION_xs__union_content_element)
+                            (*gp).choice->__contents.push_back(*d);
+                      (*g).choice->__contents.erase(c);
+                    }
+                  }
+                  else
+                  {
+                    ++c;
+                  }
+                }
+              }
+              else if ((*g).sequence && (*gp).sequence)
+              {
+                xs__seqchoice& s = *(*gp).sequence;
+                (*gp).sequence = soap_new_xs__seqchoice(schema.soap);
+                for (vector<xs__contents>::iterator c = (*g).sequence->__contents.begin(); c != (*g).sequence->__contents.end(); )
+                {
+                  if ((*c).__union == SOAP_UNION_xs__union_content_element)
+                  {
+                    (*gp).sequence->__contents.push_back(*c);
+                    ++c;
+                  }
+                  else if ((*c).__union == SOAP_UNION_xs__union_content_group)
+                  {
+                    if ((*c).__content.group->ref)
+                    {
+                      const char *token = qname_token((*c).__content.group->ref, schema.targetNamespace);
+                      if (token && !strcmp((*gp).name, token))
+                        for (vector<xs__contents>::const_iterator d = s.__contents.begin(); d != s.__contents.end(); ++d)
+                          if ((*d).__union == SOAP_UNION_xs__union_content_element)
+                            (*gp).sequence->__contents.push_back(*d);
+                      (*g).sequence->__contents.erase(c);
+                    }
+                  }
+                  else
+                  {
+                    ++c;
+                  }
+                }
+              }
               break;
             }
           }
@@ -842,13 +922,15 @@ int xs__redefine::preprocess(xs__schema &schema)
           {
             if ((*s).name && !strcmp((*st).name, (*s).name))
             {
-              *st = *s; // TODO would it be too crude to simply replace all content?
+              char *base = (*st).restriction ? (*st).restriction->base : (char*)"xs:string";
+              *st = *s;
+              (*st).restriction->base = base;
               break;
             }
           }
         }
       }
-      // redefine complexType by extension
+      // redefine complexType by extension/restriction
       for (vector<xs__complexType>::iterator ct = schemaRef->complexType.begin(); ct != schemaRef->complexType.end(); ++ct)
       {
         if ((*ct).name)
@@ -857,18 +939,102 @@ int xs__redefine::preprocess(xs__schema &schema)
           {
             if ((*c).name && !strcmp((*ct).name, (*c).name))
             {
-              if ((*c).complexContent && (*c).complexContent->extension && (*c).complexContent->extension->sequence && qname_token((*c).complexContent->extension->base, schemaRef->targetNamespace))
+              if ((*c).complexContent && (*c).complexContent->extension && qname_token((*c).complexContent->extension->base, schemaRef->targetNamespace))
               {
-                if (!(*ct).sequence)
-                  (*ct).sequence = (*c).complexContent->extension->sequence;
+                if ((*ct).all && (*c).complexContent->extension->all)
+                {
+                  (*ct).all->element.insert((*ct).all->element.end(), (*c).complexContent->extension->all->element.begin(), (*c).complexContent->extension->all->element.end()); 
+                }
+                else if ((*ct).choice && (*c).complexContent->extension->choice)
+                {
+                  (*ct).choice->__contents.insert((*ct).choice->__contents.end(), (*c).complexContent->extension->choice->__contents.begin(), (*c).complexContent->extension->choice->__contents.end()); 
+                }
+                else if ((*ct).sequence && (*c).complexContent->extension->sequence)
+                {
+                  (*ct).sequence->__contents.insert((*ct).sequence->__contents.end(), (*c).complexContent->extension->sequence->__contents.begin(), (*c).complexContent->extension->sequence->__contents.end()); 
+                }
+                else if ((*ct).group && (*c).complexContent->extension->group)
+                {
+                  if ((*ct).group->sequence && (*c).complexContent->extension->group->sequence)
+                    (*ct).group->sequence->__contents.insert((*ct).group->sequence->__contents.end(), (*c).complexContent->extension->group->sequence->__contents.begin(), (*c).complexContent->extension->group->sequence->__contents.end()); 
+                }
+                else if ((*ct).complexContent && (*ct).complexContent->extension)
+                {
+                  if ((*ct).complexContent->extension->all && (*c).complexContent->extension->all)
+                  {
+                    (*ct).complexContent->extension->all->element.insert((*ct).complexContent->extension->all->element.end(), (*c).complexContent->extension->all->element.begin(), (*c).complexContent->extension->all->element.end()); 
+                  }
+                  else if ((*ct).complexContent->extension->choice && (*c).complexContent->extension->choice)
+                  {
+                    (*ct).complexContent->extension->choice->__contents.insert((*ct).complexContent->extension->choice->__contents.end(), (*c).complexContent->extension->choice->__contents.begin(), (*c).complexContent->extension->choice->__contents.end()); 
+                  }
+                  else if ((*ct).complexContent->extension->sequence && (*c).complexContent->extension->sequence)
+                  {
+                    (*ct).complexContent->extension->sequence->__contents.insert((*ct).complexContent->extension->sequence->__contents.end(), (*c).complexContent->extension->sequence->__contents.begin(), (*c).complexContent->extension->sequence->__contents.end()); 
+                  }
+                  else if ((*ct).complexContent->extension->group && (*c).complexContent->extension->group)
+                  {
+                    if ((*ct).complexContent->extension->group->sequence && (*c).complexContent->extension->group->sequence)
+                      (*ct).complexContent->extension->group->sequence->__contents.insert((*ct).complexContent->extension->group->sequence->__contents.end(), (*c).complexContent->extension->group->sequence->__contents.begin(), (*c).complexContent->extension->group->sequence->__contents.end()); 
+                  }
+                  else
+                  {
+                    (*ct).complexContent->extension->all = (*c).complexContent->extension->all;
+                    (*ct).complexContent->extension->choice = (*c).complexContent->extension->choice;
+                    (*ct).complexContent->extension->sequence = (*c).complexContent->extension->sequence;
+                    (*ct).complexContent->extension->group = (*c).complexContent->extension->group;
+                  }
+                }
+                else if ((*ct).complexContent && (*ct).complexContent->restriction)
+                {
+                  if ((*ct).complexContent->restriction->all && (*c).complexContent->extension->all)
+                  {
+                    (*ct).complexContent->restriction->all->element.insert((*ct).complexContent->restriction->all->element.end(), (*c).complexContent->extension->all->element.begin(), (*c).complexContent->extension->all->element.end()); 
+                  }
+                  else if ((*ct).complexContent->restriction->choice && (*c).complexContent->extension->choice)
+                  {
+                    (*ct).complexContent->restriction->choice->__contents.insert((*ct).complexContent->restriction->choice->__contents.end(), (*c).complexContent->extension->choice->__contents.begin(), (*c).complexContent->extension->choice->__contents.end()); 
+                  }
+                  else if ((*ct).complexContent->restriction->sequence && (*c).complexContent->extension->sequence)
+                  {
+                    (*ct).complexContent->restriction->sequence->__contents.insert((*ct).complexContent->restriction->sequence->__contents.end(), (*c).complexContent->extension->sequence->__contents.begin(), (*c).complexContent->extension->sequence->__contents.end()); 
+                  }
+                  else if ((*ct).complexContent->restriction->group && (*c).complexContent->extension->group)
+                  {
+                    if ((*ct).complexContent->restriction->group->sequence && (*c).complexContent->restriction->group->sequence)
+                      (*ct).complexContent->restriction->group->sequence->__contents.insert((*ct).complexContent->restriction->group->sequence->__contents.end(), (*c).complexContent->extension->group->sequence->__contents.begin(), (*c).complexContent->extension->group->sequence->__contents.end()); 
+                  }
+                  else
+                  {
+                    (*ct).complexContent->restriction->all = (*c).complexContent->extension->all;
+                    (*ct).complexContent->restriction->choice = (*c).complexContent->extension->choice;
+                    (*ct).complexContent->restriction->sequence = (*c).complexContent->extension->sequence;
+                    (*ct).complexContent->restriction->group = (*c).complexContent->extension->group;
+                  }
+                }
                 else
                 {
-                  for (std::vector<xs__contents>::const_iterator e = (*c).complexContent->extension->sequence->__contents.begin(); e != (*c).complexContent->extension->sequence->__contents.end(); ++e)
-                    (*ct).sequence->__contents.push_back(*e);
+                  (*ct).all = (*c).complexContent->extension->all;
+                  (*ct).choice = (*c).complexContent->extension->choice;
+                  (*ct).sequence = (*c).complexContent->extension->sequence;
+                  (*ct).group = (*c).complexContent->extension->group;
                 }
+                (*ct).attribute.insert((*ct).attribute.begin(), (*c).complexContent->extension->attribute.begin(), (*c).complexContent->extension->attribute.end());
+              }
+              else if ((*c).complexContent && (*c).complexContent->restriction && qname_token((*c).complexContent->restriction->base, schemaRef->targetNamespace))
+              {
+                (*ct).all = (*c).complexContent->restriction->all;
+                (*ct).choice = (*c).complexContent->restriction->choice;
+                (*ct).sequence = (*c).complexContent->restriction->sequence;
+                (*ct).group = (*c).complexContent->restriction->group;
+                (*ct).attribute = (*c).complexContent->restriction->attribute;
+                (*ct).complexContent = NULL;
+              }
+              else
+              {
+                if (!Wflag)
+                  fprintf(stderr, "\nWarning: redefining complexType \"%s\" in schema \"%s\" by extension requires an extension base\n", (*ct).name, schemaRef->targetNamespace ? schemaRef->targetNamespace : "(null)");
               }
-              else if (!Wflag)
-                fprintf(stderr, "\nWarning: redefining complexType \"%s\" in schema \"%s\" by extension requires an extension base\n", (*ct).name, schemaRef->targetNamespace ? schemaRef->targetNamespace : "(null)");
               break;
             }
           }
@@ -1288,6 +1454,11 @@ void xs__attribute::mark()
   }
 }
 
+bool xs__attribute::is_used() const
+{
+  return used;
+}
+
 xs__element::xs__element()
 {
   schemaRef = NULL;
@@ -1471,7 +1642,7 @@ int xs__element::traverse(xs__schema &schema)
             elt->soap_default(schema.soap);
             elt->name = name;
             elt->form = form;
-            elt->elementPtr(this); // create element ref in substitutionsGroup
+            elt->elementPtr(this); // create element ref in substitutionGroup
             elt->schemaPtr(schemaPtr());
             elt->targetNamespace = NULL;
           }
@@ -1501,7 +1672,7 @@ int xs__element::traverse(xs__schema &schema)
                 elt->soap_default(schema.soap);
                 elt->name = name;
                 elt->form = form;
-                elt->elementPtr(this); // create element ref in substitutionsGroup
+                elt->elementPtr(this); // create element ref in substitutionGroup
                 elt->schemaPtr(schemaPtr());
                 elt->targetNamespace = NULL;
               }
@@ -1591,9 +1762,17 @@ void xs__element::mark()
       simpleTypePtr()->mark();
     if (complexTypePtr())
       complexTypePtr()->mark();
+    for (std::vector<xs__element*>::iterator i = substitutions.begin(); i != substitutions.end(); ++i)
+      if ((*i)->elementPtr())
+        (*i)->elementPtr()->mark();
   }
 }
 
+bool xs__element::is_used() const
+{
+  return used;
+}
+
 xs__simpleType::xs__simpleType()
 {
   schemaRef = NULL;
@@ -1607,11 +1786,25 @@ int xs__simpleType::traverse(xs__schema &schema)
     cerr << "   Analyzing schema simpleType '" << (name ? name : "(null)") << "'" << endl;
   schemaRef = &schema;
   if (list)
+  {
     list->traverse(schema);
+  }
   else if (restriction)
+  {
     restriction->traverse(schema);
+    if (name)
+    {
+      xs__simpleType *base = restriction->simpleTypePtr();
+      if (base)
+        base->add_restriction(schema, name);
+      else if (is_builtin_qname(restriction->base))
+        schema.builtinTypeDerivation(schema, restriction->base, name);
+    }
+  }
   else if (union_)
+  {
     union_->traverse(schema);
+  }
   return SOAP_OK;
 }
 
@@ -1674,6 +1867,26 @@ bool xs__simpleType::is_used() const
   return used;
 }
 
+void xs__simpleType::add_extension(xs__schema& schema, xsd__NCName name)
+{
+  extensions.push_back(make_qname(schema, name));
+}
+
+void xs__simpleType::add_restriction(xs__schema& schema, xsd__NCName name)
+{
+  restrictions.push_back(make_qname(schema, name));
+}
+
+const std::vector<xsd__QName>& xs__simpleType::get_extensions() const
+{
+  return extensions;
+}
+
+const std::vector<xsd__QName>& xs__simpleType::get_restrictions() const
+{
+  return restrictions;
+}
+
 xs__complexType::xs__complexType()
 {
   schemaRef = NULL;
@@ -1687,19 +1900,83 @@ int xs__complexType::traverse(xs__schema &schema)
     cerr << "   Analyzing schema complexType '" << (name ? name : "(null)") << "'" << endl;
   schemaRef = &schema;
   if (simpleContent)
+  {
     simpleContent->traverse(schema);
+    if (name)
+    {
+      if (simpleContent->extension)
+      {
+        xs__complexType *ct_base = simpleContent->extension->complexTypePtr();
+        if (ct_base)
+        {
+          ct_base->add_extension(schema, name);
+        }
+        else
+        {
+          xs__simpleType *st_base = simpleContent->extension->simpleTypePtr();
+          if (st_base)
+            st_base->add_extension(schema, name);
+          else if (is_builtin_qname(simpleContent->extension->base))
+            schema.builtinTypeDerivation(schema, simpleContent->extension->base, name);
+        }
+      }
+      else if (simpleContent->restriction)
+      {
+        xs__complexType *ct_base = simpleContent->restriction->complexTypePtr();
+        if (ct_base)
+        {
+          ct_base->add_restriction(schema, name);
+        }
+        else
+        {
+          xs__simpleType *st_base = simpleContent->restriction->simpleTypePtr();
+          if (st_base)
+            st_base->add_restriction(schema, name);
+          else if (is_builtin_qname(simpleContent->restriction->base))
+            schema.builtinTypeDerivation(schema, simpleContent->restriction->base, name);
+        }
+      }
+    }
+  }
   else if (complexContent)
+  {
     complexContent->traverse(schema);
+    if (name)
+    {
+      if (complexContent->extension)
+      {
+        xs__complexType *ct_base = complexContent->extension->complexTypePtr();
+        if (ct_base)
+          ct_base->add_extension(schema, name);
+      }
+      else if (complexContent->restriction)
+      {
+        xs__complexType *ct_base = complexContent->restriction->complexTypePtr();
+        if (ct_base)
+          ct_base->add_restriction(schema, name);
+      }
+    }
+  }
   else if (all)
+  {
     all->traverse(schema);
+  }
   else if (choice)
+  {
     choice->traverse(schema);
+  }
   else if (sequence)
+  {
     sequence->traverse(schema);
+  }
   else if (group)
+  {
     group->traverse(schema);
+  }
   else if (any)
+  {
     any->traverse(schema);
+  }
   for (vector<xs__attribute>::iterator at = attribute.begin(); at != attribute.end(); ++at)
     (*at).traverse(schema);
   for (vector<xs__attributeGroup>::iterator ag = attributeGroup.begin(); ag != attributeGroup.end(); ++ag)
@@ -1808,6 +2085,26 @@ bool xs__complexType::is_used() const
   return used;
 }
 
+void xs__complexType::add_extension(xs__schema& schema, xsd__NCName name)
+{
+  extensions.push_back(make_qname(schema, name));
+}
+
+void xs__complexType::add_restriction(xs__schema& schema, xsd__NCName name)
+{
+  restrictions.push_back(make_qname(schema, name));
+}
+
+const std::vector<xsd__QName>& xs__complexType::get_extensions() const
+{
+  return extensions;
+}
+
+const std::vector<xsd__QName>& xs__complexType::get_restrictions() const
+{
+  return restrictions;
+}
+
 int xs__simpleContent::traverse(xs__schema &schema)
 {
   if (vflag)
@@ -2499,6 +2796,9 @@ void xs__contents::mark()
 
 xs__seqchoice::xs__seqchoice()
 {
+  minOccurs = NULL;
+  maxOccurs = NULL;
+  annotation = NULL;
   schemaRef = NULL;
 }
 
@@ -2531,7 +2831,7 @@ int xs__seqchoice::traverse(xs__schema &schema)
               {
                 (*c).__content.sequence->__contents.erase(c1);
                 if (!Wflag)
-                  cerr << "\nOptimization (-O1): removed duplicate element '" << e->name << "' from nested choice/sequence" << endl;
+                  cerr << "\nOptimization: removed duplicate element '" << e->name << "' from nested choice/sequence" << endl;
               }
               else
               {
@@ -2563,7 +2863,7 @@ int xs__seqchoice::traverse(xs__schema &schema)
           if (members.find(e->name) != members.end())
           {
             __contents.erase(c);
-            cerr << "\nWarning: removed duplicate element '" << e->name << "' from nested xs:sequence/xs:choice" << endl;
+            cerr << "\nOptimization: removed duplicate element '" << e->name << "' from nested xs:sequence/xs:choice" << endl;
           }
           else
           {
