@@ -97,18 +97,20 @@ handle that is internally used by the plugin until the `soap` context is
 deleted.  For C++ proxy classes generated with soapcpp2, register the plugin
 with the `soap` member of the proxy class.
 
-The gSOAP HTTP chunked transfer mode and timeout settings are also used by the
-CURL plugin, when set, as follows:
+The gSOAP HTTP chunked transfer mode `SOAP_IO_CHUNK` and timeout settings are
+also used by the CURL plugin, when set, as follows:
 
 @code
     #include "plugin/curlapi.h"
     ...
     struct soap *soap;
     curl_global_init(CURL_GLOBAL_ALL);
-    soap = soap_new1(SOAP_IO_CHUNK);
+    soap = soap_new1(SOAP_IO_CHUNK | SOAP_XML_INDENT);
     soap_register_plugin(soap, soap_curl);
     soap->connect_timeout = 60;  // 1 minute
-    soap->transfer_timeout = 10; // 10 seconds
+    soap->send_timeout = 10;     // 10 seconds
+    soap->recv_timeout = 10;     // 10 seconds
+    soap->transfer_timeout = 20; // 20 seconds
     ...
     // client program runs
     ...
@@ -118,6 +120,29 @@ CURL plugin, when set, as follows:
     curl_global_cleanup();
 @endcode
 
+It is strongly recommended to set timeouts.  The timeout values specified here
+are just examples.  Actual values depend on the application's performance
+characteristics.
+
+Also compression is used by the CURL plugin when enabled with `SOAP_ENC_ZLIB`:
+
+@code
+    #include "plugin/curlapi.h"
+    ...
+    struct soap *soap;
+    curl_global_init(CURL_GLOBAL_ALL);
+    soap = soap_new1(SOAP_IO_CHUNK | SOAP_ENC_ZLIB | SOAP_XML_INDENT);
+    soap_register_plugin(soap, soap_curl);
+    ...
+    // client program runs
+    ...
+    soap_destroy(soap);
+    soap_end(soap);
+    soap_free(soap);
+    curl_global_cleanup();
+@endcode
+
+
 When an transmission error occurs, use `soap_curl_reset(soap)` to reset the
 plugin.  This ensures that the gSOAP IO operations are reset and will behave
 again normally.
@@ -134,7 +159,7 @@ pass it to the plugin as follows:
     curl = curl_easy_init();
     curl_easy_setopt(data->curl, CURLOPT_CONNECTTIMEOUT, 60L);
     curl_easy_setopt(data->curl, CURLOPT_TIMEOUT, 10L);
-    soap = soap_new1(SOAP_IO_CHUNK);
+    soap = soap_new1(SOAP_XML_INDENT);
     soap_register_plugin_arg(soap, soap_curl, curl);
     ...
     // client program runs
@@ -153,7 +178,7 @@ Note that C++ proxy classes generated by soapcpp2 with option `-j` have a
 @code
     #include "plugin/curlapi.h"
     ...
-    Proxy proxy(SOAP_IO_CHUNK);
+    Proxy proxy(SOAP_XML_INDENT);
     CURL *curl;
     curl_global_init(CURL_GLOBAL_ALL);
     curl = curl_easy_init();
@@ -217,7 +242,7 @@ two values:
 
     int main(int argc, char **argv)
     {
-      struct soap *soap = soap_new1(SOAP_IO_CHUNK | SOAP_XML_INDENT);
+      struct soap *soap = soap_new1(SOAP_XML_INDENT);
       double result;
       curl_global_init(CURL_GLOBAL_ALL);
       soap_register_plugin(soap, soap_curl);
@@ -335,19 +360,18 @@ soap_curl(struct soap *soap, struct soap_plugin *p, void *arg)
   p->data = (void*)SOAP_MALLOC(soap, sizeof(struct soap_curl_data));
   p->fcopy = NULL;
   p->fdelete = soap_curl_delete;
-  if (p->data)
+  if (!p->data)
+    return SOAP_EOM;
+  if (soap_curl_init(soap, (struct soap_curl_data*)p->data, (CURL*)arg))
   {
-    if (soap_curl_init(soap, (struct soap_curl_data*)p->data, (CURL*)arg))
-    {
-      SOAP_FREE(soap, p->data);
-      return SOAP_EOM;
-    }
+    SOAP_FREE(soap, p->data);
+    return SOAP_EOM;
   }
   return SOAP_OK;
 }
 
 /**
-@fn int soap_wsa_init(struct soap *soap, struct soap_wsa_data *data)
+@fn int soap_curl_init(struct soap *soap, struct soap_wsa_data *data)
 @brief Initializes plugin data.
 @param soap context
 @param[in,out] data plugin data
@@ -423,7 +447,12 @@ soap_curl_reset(struct soap *soap)
   struct soap_curl_data *data = (struct soap_curl_data*)soap_lookup_plugin(soap, soap_curl_id);
   DBGFUN("soap_curl_reset");
   if (data)
+  {
+    if (data->lst)
+      soap_end_block(soap, data->lst);
+    data->lst = NULL;
     data->active = 0;
+  }
 }
 
 /******************************************************************************\
@@ -511,14 +540,42 @@ static int soap_curl_connect_callback(struct soap *soap, const char *endpoint, c
     curl_easy_setopt(data->curl, CURLOPT_TIMEOUT, (long)soap->recv_timeout);
   else if (soap->recv_timeout < 0)
     curl_easy_setopt(data->curl, CURLOPT_TIMEOUT_MS, -(long)soap->recv_timeout/1000);
+  if (soap->proxy_host)
+  {
+    (SOAP_SNPRINTF(soap->tmpbuf, sizeof(soap->tmpbuf), strlen(soap->proxy_host) + 8 + 8), "http://%s:%d", soap->proxy_host,soap->proxy_port);
+    curl_easy_setopt(data->curl, CURLOPT_PROXY, (void*)soap->tmpbuf);
+    if (soap->proxy_userid)
+    {
+      size_t len_proxy_userid = strlen(soap->proxy_userid);
+      size_t len_proxy_passwd = soap->proxy_passwd ? strlen(soap->proxy_passwd) : 0;
+      if ((len_proxy_userid + len_proxy_passwd) * 3 + 2 < sizeof(soap->tmpbuf))
+      {
+        size_t pos = 0;
+        size_t i;
+        for (i = 0; i < len_proxy_userid; i++)
+        {
+          (SOAP_SNPRINTF(soap->tmpbuf + pos, sizeof(soap->tmpbuf) - pos, 4), "%%%02x", (int)((unsigned char)soap->proxy_userid[i]));
+          pos += 3;
+        }
+        soap->tmpbuf[pos] = ':';
+        pos++;
+        for (i = 0; i < len_proxy_passwd; i++)
+        {
+          (SOAP_SNPRINTF(soap->tmpbuf + pos, sizeof(soap->tmpbuf) - pos, 4), "%%%02x", (int)((unsigned char)soap->proxy_passwd[i]));
+          pos += 3;
+        }
+        soap->tmpbuf[pos] = '\0';
+        curl_easy_setopt(data->curl, CURLOPT_PROXYUSERPWD, (void*)soap->tmpbuf);
+      }
+    }
+  }
   soap->omode &= ~SOAP_IO;       /* reset IO modes */
   soap->omode |= SOAP_IO_BUFFER; /* buffer the output */
   soap->omode |= SOAP_ENC_PLAIN; /* no HTTP headers */
   soap->omode &= ~SOAP_ENC_ZLIB;
-  /* store data sent by engine in a blist */
-  if (!soap_alloc_block(soap))
+  /* store data sent by engine in an isolated blist */
+  if (!(data->lst = soap_alloc_block(soap)))
     return soap->error;
-  data->lst = soap->blist;
   soap->blist = soap->blist->next;
   /* activate callbacks */
   data->active = 1;
@@ -538,10 +595,12 @@ static int soap_curl_send_callback(struct soap *soap, const char *buf, size_t le
   struct soap_curl_data *data = (struct soap_curl_data*)soap_lookup_plugin(soap, soap_curl_id);
   char *blk;
   DBGFUN1("soap_curl_send_callback", "len=%zu", len);
-  if (!data || !data->lst)
+  if (!data)
     return soap->error = SOAP_PLUGIN_ERROR;
   if (!data->active)
     return data->fsend(soap, buf, len);
+  if (!data->curl || !data->lst)
+    return soap->error = SOAP_PLUGIN_ERROR;
   if (len > 0)
   {
     blk = (char*)soap_push_block(soap, data->lst, len);
@@ -563,11 +622,16 @@ static int soap_curl_prepare_init_recv_callback(struct soap *soap)
   struct soap_curl_data *data = (struct soap_curl_data*)soap_lookup_plugin(soap, soap_curl_id);
   long status;
   const char *s = NULL;
+  CURLcode res;
   DBGFUN("soap_curl_prepare_init_recv_callback");
   if (!data || !data->curl)
     return soap->error = SOAP_PLUGIN_ERROR;
-  if (!data->lst)
+  if (!data->active || !data->lst)
+  {
+    if (data->fprepareinitrecv)
+      return data->fprepareinitrecv(soap);
     return SOAP_OK;
+  }
   if ((data->mode & SOAP_IO) == SOAP_IO_CHUNK)
   {
     /* HTTP chunking mode was set with soap_init1() */
@@ -591,7 +655,7 @@ static int soap_curl_prepare_init_recv_callback(struct soap *soap)
   }
   data->hdr = curl_slist_append(data->hdr, "Expect:"); /* remove Expect: 100 */
   curl_easy_setopt(data->curl, CURLOPT_HTTPHEADER, (void*)data->hdr);
-  CURLcode res = curl_easy_perform(data->curl);
+  res = curl_easy_perform(data->curl);
   if (data->hdr)
     curl_slist_free_all(data->hdr);
   data->hdr = NULL;
@@ -610,9 +674,8 @@ static int soap_curl_prepare_init_recv_callback(struct soap *soap)
       return data->fprepareinitrecv(soap);
     return SOAP_OK;
   }
-  /* read HTTP body for error details */
-  s = soap_get_http_body(soap, NULL);
-  return soap_set_receiver_error(soap, "HTTP Error", s, soap->status);
+  /* read HTTP body for error details and return HTTP error */
+  return soap_set_receiver_error(soap, "HTTP Error", soap_http_get_body(soap, NULL), soap->status);
 }
 
 /**
@@ -647,7 +710,7 @@ static size_t soap_curl_recv_callback(struct soap *soap, char *buf, size_t size)
   struct soap_curl_data *data = (struct soap_curl_data*)soap_lookup_plugin(soap, soap_curl_id);
   size_t len;
   DBGFUN1("soap_curl_recv_callback", "size=%zu", size);
-  if (!data || !data->curl)
+  if (!data)
   {
     soap->error = SOAP_PLUGIN_ERROR;
     return 0;
@@ -756,8 +819,13 @@ static size_t soap_curl_write_callback(void *buffer, size_t size, size_t nitems,
   size_t len = size * nitems;
   char *s;
   DBGFUN2("soap_curl_write_callback", "size=%zu", size, "nitems=%zu", nitems);
-  if (!data->lst && !(data->lst = soap_alloc_block(soap)))
-    return 0;
+  if (!data->lst)
+  {
+    /* store data received in an isolated blist */
+    if (!(data->lst = soap_alloc_block(soap)))
+      return 0;
+    soap->blist = soap->blist->next;
+  }
   s = (char*)soap_push_block(soap, data->lst, len);
   if (!s)
     return 0;
