@@ -58,16 +58,16 @@ buffered message encryption modes are supported.
 An encryption and decryption algorithm and mode is selected with one of the
 following:
 
-- @ref SOAP_MEC_ENV_ENC_DES_CBC         envelope encryption with triple DES CBC
+- @ref SOAP_MEC_ENV_ENC_DES_CBC         envelope encryption with triple DES EDE CBC
 - @ref SOAP_MEC_ENV_ENC_AES256_CBC      envelope encryption with AES256 CBC
 - @ref SOAP_MEC_ENV_ENC_AES256_GCM      envelope authenticated encryption with AES256 GCM
-- @ref SOAP_MEC_ENC_DES_CBC             symmetric encryption with triple DES CBC
+- @ref SOAP_MEC_ENC_DES_CBC             symmetric encryption with triple DES EDE CBC
 - @ref SOAP_MEC_ENC_AES256_CBC          symmetric encryption with AES256 CBC
 - @ref SOAP_MEC_ENC_AES256_GCM          symmetric authenticated encryption with AES256 GCM
-- @ref SOAP_MEC_ENV_DEC_DES_CBC         envelope decryption with triple DES CBC
+- @ref SOAP_MEC_ENV_DEC_DES_CBC         envelope decryption with triple DES EDE CBC
 - @ref SOAP_MEC_ENV_DEC_AES256_CBC      envelope decryption with AES256 CBC
 - @ref SOAP_MEC_ENV_DEC_AES256_GCM      envelope authenticated decryption with AES256 GCM
-- @ref SOAP_MEC_DEC_DES_CBC             symmetric decryption with triple DES CBC
+- @ref SOAP_MEC_DEC_DES_CBC             symmetric decryption with triple DES EDE CBC
 - @ref SOAP_MEC_DEC_AES256_CBC          symmetric decryption with AES256 CBC
 - @ref SOAP_MEC_DEC_AES256_GCM          symmetric authenticated decryption with AES256 GCM
 
@@ -82,7 +82,8 @@ The mecevp engine wraps the EVP API with four new functions:
 
 - @ref soap_mec_init    to initialize the engine
 - @ref soap_mec_update  to encrypt/decrypt a message part
-- @ref soap_mec_final   to finalize encryption/decryption
+- @ref soap_mec_final   to finalize encryption/decryption, does not deallocate
+                        tne engine and buffers
 - @ref soap_mec_cleanup to deallocate the engine and buffers
 
 All cipher data is written and read in base64 format.
@@ -138,6 +139,32 @@ communicated to the recipient/reader with the message to decrypt:
     EVP_PKEY_free(pubk);
 @endcode
 
+The example given above sends the output to stdout. To save the output in a
+string use the following in C:
+
+@code
+    char *str = NULL; // string to be set to the encrypted output
+    soap->os = &str;  // for C code only 
+    if (soap_begin_send(soap)
+      ...
+    soap->os = NULL;
+    ... // use str, which is set to the encrypted output
+    soap_end(soap); // auto-deletes str
+@endcode
+
+With C++ you should use a string stream:
+
+@code
+    std::stringstream ss;
+    soap->os = &ss;
+    if (soap_begin_send(soap)
+      ...
+    soap->os = NULL;
+    ... // use ss.str()
+    soap_destroy(soap); // cleanup
+    soap_end(soap);     // cleanup
+@endcode
+
 The decryption by the recipient/reader requires the ephemeral encrypted secret
 key generated by soap_mec_begin by the sender (as set above) to decrypt the
 message using envelope decryption with SOAP_MEC_ENV_DEC_DES_CBC.
@@ -168,9 +195,34 @@ message using envelope decryption with SOAP_MEC_ENV_DEC_DES_CBC.
     EVP_PKEY_free(privk);
 @endcode 
 
-Note that the encrypted secret key can be send in the clear or stored openly,
-since only the recipient/reader will be able to decode it for use in message
-decryption.
+The example given above reads the input from stdin. To read input from a string
+use the following in C:
+
+@code
+    char *str; // string with encrupted input
+    soap->is = str;
+    if (soap_begin_recv(soap)
+      ...
+    soap->is = NULL;
+    soap_end(soap); // cleanup
+@endcode
+
+With C++ you should use a string stream:
+
+@code
+    std::stringstream ss;
+    ss.str(...); // string with encrypted input
+    soap->is = &ss;
+    if (soap_begin_recv(soap)
+      ...
+    soap->is = NULL;
+    soap_destroy(soap); // cleanup
+    soap_end(soap);     // cleanup
+@endcode
+
+Note that the encrypted secret key can be sent in the clear or stored openly,
+since only the recipient/reader will be able to decode it (with its private
+key) and use it for message decryption.
 
 Symmetric encryption and decryption can be used if both parties can safely
 share a secret symmetric key that no other party has access to. We use
@@ -737,7 +789,7 @@ soap_mec_size(int alg, SOAP_MEC_KEY_TYPE *pkey)
   switch (alg & SOAP_MEC_ALGO & ~SOAP_MEC_GCM)
   {
     case SOAP_MEC_DES_CBC:
-      return 20;
+      return 24;
     case SOAP_MEC_AES128_CBC: /* CBC and GCM */
       return 16;
     case SOAP_MEC_AES192_CBC: /* CBC and GCM */
@@ -1051,7 +1103,7 @@ soap_mec_upd_dec(struct soap *soap, struct soap_mec_data *data, const char **s,
         switch (data->alg & SOAP_MEC_MASK & ~SOAP_MEC_ALGO)
         {
           case SOAP_MEC_ENV_DEC:
-            ok = EVP_OpenInit(data->ctx, data->type, data->key, data->keylen, (unsigned char*)data->buf, (EVP_PKEY*)data->pkey);
+            ok = EVP_OpenInit(data->ctx, data->type, (unsigned char*)data->key, data->keylen, (unsigned char*)data->buf, (EVP_PKEY*)data->pkey);
             EVP_CIPHER_CTX_set_padding(data->ctx, 0); /* disable padding check when decrypting for interop */
             DBGLOG(TEST, SOAP_MESSAGE(fdebug, "EVP_OpenInit ok=%d\n", ok));
             break;
