@@ -55,6 +55,7 @@ The threads.h and threads.c code define the following portable API:
 - THREAD_TYPE            portable thread type
 - THREAD_ID              returns current thread ID of type THREAD_TYPE*
 - THREAD_CREATE(t,f,a)   start thread (THREAD_TYPE*)t for f(a), return 0 if OK
+- THREAD_CREATEX(t,f,a)  Windows only: start joinable thread (THREAD_TYPE*)t for f(a), return 0 if OK
 - THREAD_DETACH(t)       detach thread (THREAD_TYPE*)t
 - THREAD_JOIN(t)         wait to join (THREAD_TYPE*)t
 - THREAD_EXIT            exit the current thread
@@ -103,11 +104,13 @@ The threads.h and threads.c code define the following portable API:
 #if defined(WIN32)
 # define THREAD_TYPE		HANDLE
 # define THREAD_ID		GetCurrentThreadId()
-# define THREAD_CREATE(x,y,z)	((*(x) = (HANDLE)_beginthread((_beginthread_proc_type)(y), 8*4096, (z))) == (HANDLE)-1L)
+# define THREAD_CREATE(x,y,z)	((*(x) = (HANDLE)_beginthread((void(__cdecl*)(void*))(y), 8*4096, (z))) == (HANDLE)-1L)
+# define THREAD_CREATEX(x,y,z)  ((*(x) = (HANDLE)_beginthreadex((void(__cdecl*)(void*))(y), 8*4096, (z))) == (HANDLE)-1L)
+# define THREAD_CLOSE(x)	CloseHandle(x)
 # define THREAD_DETACH(x)	
-# define THREAD_JOIN(x)		WaitForSingleObject((x), INFINITE)
+# define THREAD_JOIN(x)		(WaitForSingleObject((x), INFINITE) == (DWORD)0xFFFFFFFF)
 # define THREAD_EXIT		_endthread()
-# define THREAD_CANCEL(x)       TerminateThread(x, 0)
+# define THREAD_CANCEL(x)       (TerminateThread(x, 0) == 0)
 # define MUTEX_TYPE		HANDLE
 # define MUTEX_INITIALIZER	NULL
 # define MUTEX_SETUP(x)		(x) = CreateMutex(NULL, FALSE, NULL)
@@ -138,6 +141,8 @@ SOAP_FMAC1 int SOAP_FMAC2 emulate_pthread_cond_wait(COND_TYPE*, MUTEX_TYPE*);
 # define THREAD_TYPE		pthread_t
 # define THREAD_ID		pthread_self()
 # define THREAD_CREATE(x,y,z)	pthread_create((x), NULL, (y), (z))
+# define THREAD_CREATEX(x,y,z)	pthread_create((x), NULL, (y), (z))
+# define THREAD_CLOSE(x)
 # define THREAD_DETACH(x)	pthread_detach((x))
 # define THREAD_JOIN(x)		pthread_join((x), NULL)
 # define THREAD_EXIT		pthread_exit(NULL)
