@@ -1,7 +1,7 @@
 /*
         webserver.c
 
-        Example stand-alone gSOAP Web server based on the gSOAP HTTP GET plugin.
+        Example stand-alone gSOAP Web server based on gSOAP plugins.
         This is a small but fully functional (embedded) Web server for serving
         static and dynamic pages and SOAP/XML responses over HTTP/HTTPS.
 
@@ -133,6 +133,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "httpget.h"
 #include "httppost.h"
 #include "httpform.h"
+/* #include "httppipe.h" */ /* optionally enable HTTP pipelining at the cost of increased memory usage */
 #include "logging.h"
 #include "threads.h"
 #ifdef WITH_OPENSSL
@@ -146,6 +147,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #define AUTH_USERID "admin"     /* user ID to access admin pages */
 #define AUTH_PASSWD "guest"     /* user pw to access admin pages */
 
+void sigpipe_handle(int x) { }
+
 /******************************************************************************\
  *
  *      Thread pool and request queue
@@ -241,7 +244,6 @@ int calcpost(struct soap*);
 int info(struct soap*);
 int html_hbar(struct soap*, const char*, size_t, size_t, unsigned long);
 int html_hist(struct soap*, const char*, size_t, size_t, size_t, const char**, size_t*, size_t);
-void sigpipe_handle(int); /* SIGPIPE handler: Unix/Linux only */
 int CRYPTO_thread_setup();
 void CRYPTO_thread_cleanup();
 
@@ -269,6 +271,7 @@ int main(int argc, char **argv)
   struct soap soap;
   SOAP_SOCKET master;
   int port = 0;
+  int backlog = BACKLOG;
 
   start = time(NULL);
 
@@ -280,6 +283,8 @@ int main(int argc, char **argv)
     port = atol(options[OPTION_port].value);
   if (!port)
     port = 8080;
+  if (options[OPTION_i].selected)
+    backlog = 1;
   fprintf(stderr, "Starting Web server on port %d\n", port);
   /* if the port is an odd number, the Web server uses HTTPS only */
   if (port % 2)
@@ -291,13 +296,13 @@ int main(int argc, char **argv)
   fprintf(stderr, "[Note: http://localhost:%d for settings, login: '"AUTH_USERID"' and '"AUTH_PASSWD"']\n", port);
   fprintf(stderr, "[Note: you should enable Linux/Unix SIGPIPE handlers to avoid broken pipe]\n");
 
-  /* Init SSL (can skip or call multiple times, engien inits automatically) */
+  /* Init SSL (can skip or call multiple times, engine inits automatically) */
   soap_ssl_init();
-  /* soap_ssl_noinit(); call this first if SSL is initialized elsewhere */
+  /* soap_ssl_noinit(); call this first if SSL is initialized elsewhere in an application */
 
-  soap_init2(&soap, SOAP_IO_KEEPALIVE, SOAP_IO_DEFAULT);
+  soap_init(&soap);
 
-  /* set up lSSL ocks */
+  /* set up SSL locks */
   if (CRYPTO_thread_setup())
   {
     fprintf(stderr, "Cannot setup thread mutex\n");
@@ -343,9 +348,16 @@ int main(int argc, char **argv)
   /* Register HTTP form POST plugin (MUST be registered AFTER the httppost plugin) */
   if (soap_register_plugin_arg(&soap, http_form, (void*)http_form_handler))
     soap_print_fault(&soap, stderr);
+#ifdef HTTPPIPE_H
+  /* Register HTTP PIPE plugin */
+  if (soap_register_plugin(&soap, http_pipe))
+    soap_print_fault(&soap, stderr);
+#endif
+#ifdef LOGGING_H
   /* Register logging plugin */
   if (soap_register_plugin(&soap, logging))
     soap_print_fault(&soap, stderr);
+#endif
 #ifdef HTTPDA_H
   /* Register HTTP Digest Authentication plugin */
   if (soap_register_plugin(&soap, http_da))
@@ -355,7 +367,7 @@ int main(int argc, char **argv)
   /* soap.accept_flags = SO_NOSIGPIPE; */       /* some systems like this */
   /* soap.socket_flags = MSG_NOSIGNAL; */       /* others need this */
   /* signal(SIGPIPE, sigpipe_handle); */        /* and some older Unix systems may require a sigpipe handler */
-  master = soap_bind(&soap, NULL, port, BACKLOG);
+  master = soap_bind(&soap, NULL, port, backlog);
   if (!soap_valid_socket(master))
   {
     soap_print_fault(&soap, stderr);
@@ -388,15 +400,10 @@ void server_loop(struct soap *soap)
     SOAP_SOCKET sock;
     int newpoolsize;
     
-    soap->cookie_domain = options[OPTION_d].value; /* set domain of this server */
-    soap->cookie_path = options[OPTION_p].value;   /* set root path of the cookies */
-    soap_set_cookie(soap, "visit", "true", NULL, NULL); /* use global domain/path */
-    soap_set_cookie_expire(soap, "visit", 600, NULL, NULL); /* max-age is 600 seconds to expire */
-
     if (options[OPTION_c].selected)
       soap_set_omode(soap, SOAP_IO_CHUNK); /* use chunked HTTP content (fast) */
     if (options[OPTION_k].selected)
-      soap_set_omode(soap, SOAP_IO_KEEPALIVE);
+      soap_set_mode(soap, SOAP_IO_KEEPALIVE); /* HTTP persistent connections */
     if (options[OPTION_t].value)
       soap->send_timeout = soap->recv_timeout = atol(options[OPTION_t].value);
     if (options[OPTION_s].value)
@@ -429,7 +436,6 @@ void server_loop(struct soap *soap)
         fprintf(stderr, "Waiting for thread %d to terminate...\n", job);
         THREAD_JOIN(tids[job]);
         fprintf(stderr, "Thread %d has stopped\n", job);
-        soap_done(soap_thr[job]);
         free(soap_thr[job]);
       }
 
@@ -449,7 +455,8 @@ void server_loop(struct soap *soap)
         soap_thr[job]->user = (void*)(long)job; /* int to ptr */
         
         fprintf(stderr, "Starting thread %d\n", job);
-        THREAD_CREATE(&tids[job], (void*(*)(void*))process_queue, (void*)soap_thr[job]);
+        while (THREAD_CREATE(&tids[job], (void*(*)(void*))process_queue, (void*)soap_thr[job]))
+          sleep(1);
       }
 
       poolsize = job;
@@ -477,7 +484,12 @@ void server_loop(struct soap *soap)
     }
 
     if (options[OPTION_v].selected)
-      fprintf(stderr, "Request #%d accepted on socket %d connected from IP %d.%d.%d.%d\n", req, sock, (int)(soap->ip>>24)&0xFF, (int)(soap->ip>>16)&0xFF, (int)(soap->ip>>8)&0xFF, (int)soap->ip&0xFF);
+    {
+      if (soap->ip)
+        fprintf(stderr, "Request #%d accepted on socket %d connected from %s IPv4=%d.%d.%d.%d\n", req, sock, soap->host, (int)(soap->ip>>24)&0xFF, (int)(soap->ip>>16)&0xFF, (int)(soap->ip>>8)&0xFF, (int)soap->ip&0xFF);
+      else
+        fprintf(stderr, "Request #%d accepted on socket %d connected from %s IPv6=%.8x%.8x%.8x%.8x\n", req, sock, soap->host, soap->ip6[0], soap->ip6[1], soap->ip6[2], soap->ip6[3]);
+    }
 
     if (poolsize > 0)
     {
@@ -574,7 +586,9 @@ void *process_request(void *soap)
     soap_print_fault(tsoap, stderr);
   }
   else if (options[OPTION_v].selected)
+  {
     fprintf(stderr, "Thread %d completed\n", (int)(long)tsoap->user);
+  }
 
   soap_destroy(tsoap);
   soap_end(tsoap);
@@ -598,7 +612,8 @@ void *process_queue(void *soap)
   {
     tsoap->socket = dequeue();
     if (!soap_valid_socket(tsoap->socket))
-    { if (options[OPTION_v].selected)
+    {
+      if (options[OPTION_v].selected)
         fprintf(stderr, "Thread %d terminating\n", (int)(long)tsoap->user);
       break;
     }
@@ -639,12 +654,13 @@ void enqueue(SOAP_SOCKET sock)
   int next;
   int ret;
 
-  if ((ret = MUTEX_LOCK(queue_lock)))
+  if ((ret = MUTEX_LOCK(queue_lock)) != 0)
     fprintf(stderr, "MUTEX_LOCK error %d\n", ret);
 
   next = (tail + 1) % MAX_QUEUE;
-  if (head == next)
-    COND_WAIT(queue_notfull, queue_lock);
+  while (head == next)
+    if ((ret = COND_WAIT(queue_notfull, queue_lock)) != 0)
+      fprintf(stderr, "COND_WAIT error %d\n", ret);
 
   queue[tail] = sock;
   tail = next;
@@ -652,7 +668,7 @@ void enqueue(SOAP_SOCKET sock)
   if (options[OPTION_v].selected)
     fprintf(stderr, "enqueue(%d)\n", sock);
 
-  if ((ret = COND_SIGNAL(queue_notempty)))
+  if ((ret = COND_SIGNAL(queue_notempty)) != 0)
     fprintf(stderr, "COND_SIGNAL error %d\n", ret);
 
   if ((ret = MUTEX_UNLOCK(queue_lock)))
@@ -664,11 +680,11 @@ SOAP_SOCKET dequeue()
   SOAP_SOCKET sock;
   int ret;
 
-  if ((ret = MUTEX_LOCK(queue_lock)))
+  if ((ret = MUTEX_LOCK(queue_lock)) != 0)
     fprintf(stderr, "MUTEX_LOCK error %d\n", ret);
 
-  if (head == tail)
-    if ((ret = COND_WAIT(queue_notempty, queue_lock)))
+  while (head == tail)
+    if ((ret = COND_WAIT(queue_notempty, queue_lock)) != 0)
       fprintf(stderr, "COND_WAIT error %d\n", ret);
 
   sock = queue[head];
@@ -760,7 +776,8 @@ int http_GET_handler(struct soap *soap)
     soap_set_omode(soap, SOAP_IO_STORE); */ /* if not chunking we MUST buffer entire content when returning HTML pages to determine content length */
 #ifdef WITH_ZLIB
   if (options[OPTION_z].selected && soap->zlib_out == SOAP_ZLIB_GZIP) /* client accepts gzip */
-  { soap_set_omode(soap, SOAP_ENC_ZLIB); /* we can compress content (gzip) */
+  {
+    soap_set_omode(soap, SOAP_ENC_ZLIB); /* we can compress content (gzip) */
     soap->z_level = 9; /* best compression */
   }
   else
@@ -788,9 +805,24 @@ int http_GET_handler(struct soap *soap)
   if (!strncmp(soap->path, "/calc?", 6))
     return calcget(soap);
   if (!strncmp(soap->path, "/genivia", 8))
-  { (SOAP_SNPRINTF(soap->endpoint, sizeof(soap->endpoint), strlen(soap->path) + 10), "http://genivia.com%s", soap->path + 8); /* redirect */
+  {
+    (SOAP_SNPRINTF(soap->endpoint, sizeof(soap->endpoint), strlen(soap->path) + 10), "http://genivia.com%s", soap->path + 8); /* redirect */
     return 307; /* Temporary Redirect */
   }
+  if (!strncmp(soap->path, "/product?SKU=", 13))
+  {
+    struct ns__record record;
+    soap_default_ns__record(soap, &record);
+    record.SKU = atoi(soap->path + 13);
+    record.product_name = (char*)"widgets";
+    record.in_store = 42;
+    soap->http_content = "text/xml";
+    if (soap_response(soap, SOAP_FILE)
+     || soap_put_ns__record(soap, &record, "ns:record", NULL)
+     || soap_end_send(soap))
+      return soap_closesock(soap);
+    return SOAP_OK;
+  }
   /* Check requestor's authentication: */
   if (check_authentication(soap))
     return 401; /* HTTP not authorized */
@@ -805,13 +837,16 @@ int http_GET_handler(struct soap *soap)
 }
 
 int check_authentication(struct soap *soap)
-{ if (soap->userid && soap->passwd)
-  { if (!strcmp(soap->userid, AUTH_USERID) && !strcmp(soap->passwd, AUTH_PASSWD))
+{
+  if (soap->userid && soap->passwd)
+  {
+    if (!strcmp(soap->userid, AUTH_USERID) && !strcmp(soap->passwd, AUTH_PASSWD))
       return SOAP_OK;
   }
 #ifdef HTTPDA_H
   else if (soap->authrealm && soap->userid)
-  { if (!strcmp(soap->authrealm, AUTH_REALM) && !strcmp(soap->userid, AUTH_USERID))
+  {
+    if (!strcmp(soap->authrealm, AUTH_REALM) && !strcmp(soap->userid, AUTH_USERID))
       if (!http_da_verify_get(soap, (char*)AUTH_PASSWD))
         return SOAP_OK;
   }
@@ -834,9 +869,17 @@ int http_PUT_handler(struct soap *soap)
   /* Note: soap->path always starts with '/' */
   if (!strcmp(soap->path, "/person.xml"))
   {
-    /* in this example we actually do not save the data as a file person.xml, but we could! */
-    const char *data = soap_get_http_body(soap, NULL);
-    return 202; /* HTTP accepted */
+    /* in this example we actually do not save the XML data as a file person.xml, but we could! */
+    const char *data = soap_http_get_body(soap, NULL);
+    (void)soap_end_recv(soap);
+    return data ? 202 : 500; /* HTTP accepted or error when no data was received */
+  }
+  if (!strcmp(soap->path, "/product"))
+  {
+    /* in this example we actually do not save the record data, but we could! */
+    struct ns__record *record = soap_get_ns__record(soap, NULL, "ns:record", NULL);
+    (void)soap_end_recv(soap);
+    return record ? 202 : 500; /* HTTP accepted */
   }
   return 404; /* HTTP not found */
 }
@@ -856,14 +899,28 @@ int http_POST_handler(struct soap *soap)
 #endif
   /* Use soap->path (from request URL) to determine request: */
   if (options[OPTION_v].selected)
-    fprintf(stderr, "HTTP POST Request: %s\n", soap->endpoint);
+    fprintf(stderr, "HTTP Request: %s\n", soap->endpoint);
   /* Note: soap->path always starts with '/' */
   if (!strcmp(soap->path, "/person.xml"))
   {
     /* in this example we actually do not save the data as a file person.xml, but we could! */
-    const char *data = soap_get_http_body(soap, NULL);
+    const char *data = soap_http_get_body(soap, NULL);
+    (void)soap_end_recv(soap);
     return copy_file(soap, "person.xml", "text/xml");
   }
+  if (!strcmp(soap->path, "/product"))
+  {
+    /* in this example we actually do not save the record data, but we could! */
+    struct ns__record *record = soap_get_ns__record(soap, NULL, "ns:record", NULL);
+    if (!record || soap_end_recv(soap))
+      return 500;
+    /* copy ns__record received to response message */
+    if (soap_response(soap, SOAP_FILE)
+     || soap_put_ns__record(soap, record, "ns:record", NULL)
+     || soap_end_send(soap))
+      return soap_closesock(soap);
+    return SOAP_OK;
+  }
   return 404; /* HTTP not found */
 }
 
@@ -913,23 +970,27 @@ int http_form_handler(struct soap *soap)
 \******************************************************************************/
 
 int copy_file(struct soap *soap, const char *name, const char *type)
-{ FILE *fd;
+{
+  FILE *fd;
   size_t r;
   fd = fopen(name, "rb"); /* open file to copy */
   if (!fd)
     return 404; /* return HTTP not found */
   soap->http_content = type;
   if (soap_response(soap, SOAP_FILE)) /* OK HTTP response header */
-  { soap_end_send(soap);
+  {
+    soap_end_send(soap);
     fclose(fd);
     return soap->error;
   }
   for (;;)
-  { r = fread(soap->tmpbuf, 1, sizeof(soap->tmpbuf), fd);
+  {
+    r = fread(soap->tmpbuf, 1, sizeof(soap->tmpbuf), fd);
     if (!r)
       break;
     if (soap_send_raw(soap, soap->tmpbuf, r))
-    { soap_end_send(soap);
+    {
+      soap_end_send(soap);
       fclose(fd);
       return soap->error;
     }
@@ -945,14 +1006,17 @@ int copy_file(struct soap *soap, const char *name, const char *type)
 \******************************************************************************/
 
 int calcget(struct soap *soap)
-{ int o = 0, a = 0, b = 0, val;
+{
+  int o = 0, a = 0, b = 0, val;
   char buf[256];
-  char *s = query(soap); /* get argument string from URL ?query string */
+  char *s = soap_query(soap); /* get argument string from URL ?query string */
   while (s)
-  { char *key = query_key(soap, &s); /* decode next query string key */
-    char *val = query_val(soap, &s); /* decode next query string value (if any) */
+  {
+    char *key = soap_query_key(soap, &s); /* decode next query string key */
+    char *val = soap_query_val(soap, &s); /* decode next query string value (if any) */
     if (key && val)
-    { if (!strcmp(key, "o"))
+    {
+      if (!strcmp(key, "o"))
         o = val[0];
       else if (!strcmp(key, "a"))
         a = strtol(val, NULL, 10);
@@ -961,7 +1025,8 @@ int calcget(struct soap *soap)
     }
   }
   switch (o)
-  { case 'a':
+  {
+    case 'a':
       val = a + b;
       break;
     case 's':
@@ -990,14 +1055,17 @@ int calcget(struct soap *soap)
 \******************************************************************************/
 
 int calcpost(struct soap *soap)
-{ int o = 0, a = 0, b = 0, val;
+{
+  int o = 0, a = 0, b = 0, val;
   char buf[256];
-  char *s = form(soap); /* get form data from body */
+  char *s = soap_http_get_form(soap); /* get form data from body */
   while (s)
-  { char *key = query_key(soap, &s); /* decode next key */
-    char *val = query_val(soap, &s); /* decode next value (if any) */
+  {
+    char *key = soap_query_key(soap, &s); /* decode next key */
+    char *val = soap_query_val(soap, &s); /* decode next value (if any) */
     if (key && val)
-    { if (!strcmp(key, "o"))
+    {
+      if (!strcmp(key, "o"))
         o = val[0];
       else if (!strcmp(key, "a"))
         a = strtol(val, NULL, 10);
@@ -1006,7 +1074,8 @@ int calcpost(struct soap *soap)
     }
   }
   switch (o)
-  { case 'a':
+  {
+    case 'a':
       val = a + b;
       break;
     case 's':
@@ -1035,12 +1104,15 @@ int calcpost(struct soap *soap)
 \******************************************************************************/
 
 int f__form1(struct soap *soap)
-{ int o = 0, a = 0, b = 0, val;
+{
+  int o = 0, a = 0, b = 0, val;
   char buf[256];
   struct soap_multipart *content;
   for (content = soap->mime.list; content; content = content->next)
-  { if (content->id && content->ptr)
-    { /* may have to check content->encoding to convert data when necessary! */
+  {
+    if (content->id && content->ptr)
+    {
+      /* may have to check content->encoding to convert data when necessary! */
       if (!strcmp(content->id, "o"))
         o = content->ptr[0];
       else if (!strcmp(content->id, "a"))
@@ -1050,7 +1122,8 @@ int f__form1(struct soap *soap)
     }
   }
   switch (o)
-  { case 'a':
+  {
+    case 'a':
       val = a + b;
       break;
     case 's':
@@ -1073,11 +1146,14 @@ int f__form1(struct soap *soap)
 }
 
 int f__form2(struct soap *soap, struct f__formResponse *response)
-{ int o = 0, a = 0, b = 0;
+{
+  int o = 0, a = 0, b = 0;
   struct soap_multipart *content;
   for (content = soap->mime.list; content; content = content->next)
-  { if (content->id && content->ptr)
-    { /* may have to check content->encoding to convert data when necessary! */
+  {
+    if (content->id && content->ptr)
+    {
+      /* may have to check content->encoding to convert data when necessary! */
       if (!strcmp(content->id, "o"))
         o = content->ptr[0];
       else if (!strcmp(content->id, "a"))
@@ -1087,7 +1163,8 @@ int f__form2(struct soap *soap, struct f__formResponse *response)
     }
   }
   switch (o)
-  { case 'a':
+  {
+    case 'a':
       response->result = a + b;
       break;
     case 's':
@@ -1112,14 +1189,16 @@ int f__form2(struct soap *soap, struct f__formResponse *response)
 \******************************************************************************/
 
 int info(struct soap *soap)
-{ struct http_get_data *getdata;
+{
+  size_t stat_get, stat_post, stat_fail, *hist_min, *hist_hour, *hist_day;
   size_t stat_sent, stat_recv;
   const char *t0, *t1, *t2, *t3, *t4, *t5, *t6, *t7;
-  char buf[4096]; /* buffer large enough to hold parts of HTML content */
+  char buf[4096]; /* a small buffer that is large enough to hold parts of HTML content */
   struct soap_plugin *p;
   time_t now = time(NULL), elapsed = now - start;
+  struct tm T;
   query_options(soap, options);
-  if (soap->omode & SOAP_IO_KEEPALIVE)
+  if ((soap->omode & SOAP_IO_KEEPALIVE))
     t0 = "<td align='center' bgcolor='green'>YES</td>";
   else
     t0 = "<td align='center' bgcolor='red'>NO</td>";
@@ -1135,26 +1214,30 @@ int info(struct soap *soap)
   t2 = "<td align='center' bgcolor='blue'>N/A</td>";
 #endif
   if (secure)
-  { t3 = "<td align='center' bgcolor='green'>YES</td>";
+  {
+    t3 = "<td align='center' bgcolor='green'>YES</td>";
     if (soap->imode & SOAP_ENC_SSL)
       t4 = "<td align='center' bgcolor='green'>PASS</td>";
     else
       t4 = "<td align='center' bgcolor='red'>FAIL</td>";
   }
   else
-  { t3 = "<td align='center' bgcolor='red'>NO</td>";
+  {
+    t3 = "<td align='center' bgcolor='red'>NO</td>";
     t4 = "<td align='center' bgcolor='blue'>N/A</td>";
   }
 #ifdef WITH_ZLIB
   if (options[OPTION_z].selected)
-  { t5 = "<td align='center' bgcolor='green'>YES</td>";
+  {
+    t5 = "<td align='center' bgcolor='green'>YES</td>";
     if (soap->omode & SOAP_ENC_ZLIB)
       t6 = "<td align='center' bgcolor='green'>PASS</td>";
     else
       t6 = "<td align='center' bgcolor='yellow'>WAIT</td>";
   }
   else
-  { t5 = "<td align='center' bgcolor='red'>NO</td>";
+  {
+    t5 = "<td align='center' bgcolor='red'>NO</td>";
     t6 = "<td align='center' bgcolor='blue'>N/A</td>";
   }
 #else
@@ -1165,10 +1248,16 @@ int info(struct soap *soap)
     t7 = "<td align='center' bgcolor='green'>YES</td>";
   else
     t7 = "<td align='center' bgcolor='red'>NO</td>";
+#ifdef WITH_COOKIES
+  soap->cookie_domain = options[OPTION_d].value; /* set domain of this server */
+  soap->cookie_path = options[OPTION_p].value;   /* set root path of the cookies */
+  soap_set_cookie(soap, "visit", "true", NULL, NULL); /* use global domain/path */
+  soap_set_cookie_expire(soap, "visit", 600, NULL, NULL); /* max-age is 600 seconds to expire */
+#endif
   if (soap_response(soap, SOAP_HTML))
     return soap->error;
-  (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "\
-<html>\
+  if (soap_send(soap, 
+"<html>\
 <head>\
 <meta name='Author' content='Robert A. van Engelen'>\
 <meta name='Generator' content='gSOAP'>\
@@ -1177,14 +1266,17 @@ int info(struct soap *soap)
 <title>gSOAP Web Server Administration</title>\
 </head>\
 <body bgcolor='#FFFFFF'>\
-<h1>gSOAP Web Server Administration</h1>\
-<p/>Server endpoint=%s client agent IP=%d.%d.%d.%d\
-<h2>Registered Plugins</h2>\
-", soap->endpoint, (int)(soap->ip>>24)&0xFF, (int)(soap->ip>>16)&0xFF, (int)(soap->ip>>8)&0xFF, (int)soap->ip&0xFF);
-  if (soap_send(soap, buf))
+<h1>gSOAP Web Server Administration</h1>"))
+    return soap->error;
+  if (soap->ip)
+    (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "<p/>Server endpoint=%s client agent %s with IPv4=%d.%d.%d.%d", soap->endpoint, soap->host, (int)(soap->ip>>24)&0xFF, (int)(soap->ip>>16)&0xFF, (int)(soap->ip>>8)&0xFF, (int)soap->ip&0xFF);
+  else
+    (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "<p/>Server endpoint=%s client agent %s with IPv6=%.8x%.8x%.8x%.8x", soap->endpoint, soap->host, soap->ip6[0], soap->ip6[1], soap->ip6[2], soap->ip6[3]);
+  if (soap_send(soap, buf) || soap_send(soap, "<h2>Registered Plugins</h2>"))
     return soap->error;
   for (p = soap->plugins; p; p = p->next)
-  { if (soap_send(soap, p->id)
+  {
+    if (soap_send(soap, p->id)
      || soap_send(soap, "<br/>"))
       return soap->error;
   }
@@ -1215,37 +1307,33 @@ int info(struct soap *soap)
 </table>", t0, t1, t2, t3, t4, t5, t6, t7);
   if (soap_send(soap, buf))
     return soap->error;
-  getdata = (struct http_get_data*)soap_lookup_plugin(soap, http_get_id);
-  soap_get_logging_stats(soap, &stat_sent, &stat_recv);
+  soap_http_get_stats(soap, &stat_get, &stat_post, &stat_fail, &hist_min, &hist_hour, &hist_day);
+  soap_logging_stats(soap, &stat_sent, &stat_recv);
   soap_send(soap, "<h2>Usage Statistics</h2>");
-  if (getdata)
-  { html_hbar(soap, "HTTP&nbsp;GET", 120, getdata->stat_get, 0x0000FF);
-    html_hbar(soap, "HTTP&nbsp;POST", 120, getdata->stat_post, 0x00FF00);
-    html_hbar(soap, "HTTP&nbsp;FAIL", 120, getdata->stat_fail, 0xFF0000);
-  }
+  html_hbar(soap, "HTTP&nbsp;GET", 120, stat_get, 0x0000FF);
+  html_hbar(soap, "HTTP&nbsp;POST", 120, stat_post, 0x00FF00);
+  html_hbar(soap, "HTTP&nbsp;FAIL", 120, stat_fail, 0xFF0000);
   html_hbar(soap, "SENT(kB)", 120, stat_sent/1024, 0x00FFFF);
   html_hbar(soap, "RECV(kB)", 120, stat_recv/1024, 0x00FFFF);
   if (elapsed > 0)
-  { html_hbar(soap, "SENT(kB/s)", 120, stat_sent/elapsed/1024, 0x00FFFF);
+  {
+    html_hbar(soap, "SENT(kB/s)", 120, stat_sent/elapsed/1024, 0x00FFFF);
     html_hbar(soap, "RECV(kB/s)", 120, stat_recv/elapsed/1024, 0x00FFFF);
   }
-  if (getdata)
-  { struct tm T;
-    T.tm_min = 99;
-    T.tm_hour = 99;
-    T.tm_yday = 999;
+  T.tm_min = 99;
+  T.tm_hour = 99;
+  T.tm_yday = 999;
 #ifdef HAVE_LOCALTIME_R
-    localtime_r(&now, &T);
+  localtime_r(&now, &T);
 #else
-    T = *localtime(&now);
+  T = *localtime(&now);
 #endif
-    soap_send(soap, "<h2>Requests by the Minute</h2>");
-    html_hist(soap, "Minute", 12, 0, 60, minutes, getdata->min, T.tm_min);
-    soap_send(soap, "<h2>Requests by the Hour</h2>");
-    html_hist(soap, "Hour", 30, 0, 24, hours, getdata->hour, T.tm_hour);
-    soap_send(soap, "<h2>Requests by Day of the Year</h2>");
-    html_hist(soap, "Day", 2, 0, 365, NULL, getdata->day, T.tm_yday);
-  }
+  soap_send(soap, "<h2>Requests by the Minute</h2>");
+  html_hist(soap, "Minute", 12, 0, 60, minutes, hist_min, T.tm_min);
+  soap_send(soap, "<h2>Requests by the Hour</h2>");
+  html_hist(soap, "Hour", 30, 0, 24, hours, hist_hour, T.tm_hour);
+  soap_send(soap, "<h2>Requests by Day of the Year</h2>");
+  html_hist(soap, "Day", 2, 0, 365, NULL, hist_day, T.tm_yday);
   soap_send(soap, "\
 <p/>This page will automatically reload every minute to refresh the statistics.\
 <br/><br/><br/><img src='favicon.gif' align='absmiddle'>Powered by gSOAP\
@@ -1255,12 +1343,15 @@ int info(struct soap *soap)
 }
 
 static size_t html_scaled(char *buf, size_t max, size_t len)
-{ if (len > 1000000)
-  { (SOAP_SNPRINTF(buf, max, 39), "%.2f&#183;10<sup>6</sup>", (float)len/1000000.0);
+{
+  if (len > 1000000)
+  {
+    (SOAP_SNPRINTF(buf, max, 39), "%.2f&#183;10<sup>6</sup>", (float)len/1000000.0);
     return len / 1000000;
   }
   if (len > 1000)
-  { (SOAP_SNPRINTF(buf, max, 39), "%.2f&#183;10<sup>3</sup>", (float)len/1000.0);
+  {
+    (SOAP_SNPRINTF(buf, max, 39), "%.2f&#183;10<sup>3</sup>", (float)len/1000.0);
     return len / 1000;
   }
   (SOAP_SNPRINTF(buf, max, 20), "%lu", (unsigned long)len);
@@ -1268,7 +1359,8 @@ static size_t html_scaled(char *buf, size_t max, size_t len)
 }
 
 int html_hbar(struct soap *soap, const char *title, size_t pix, size_t len, unsigned long color)
-{ char buf[4096]; /* buffer large enough to hold parts of HTML content */
+{
+  char buf[4096]; /* buffer large enough to hold parts of HTML content */
   char lab[40];
   len = html_scaled(lab, sizeof(lab), len);
   (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "\
@@ -1294,17 +1386,20 @@ int html_hbar(struct soap *soap, const char *title, size_t pix, size_t len, unsi
 }
 
 int html_hist(struct soap *soap, const char *title, size_t barwidth, size_t height, size_t num, const char **key, size_t *val, size_t highlight)
-{ char buf[4096]; /* buffer large enough to hold HTML content */
+{
+  char buf[4096]; /* buffer large enough to hold HTML content */
   char lab[40];
   size_t i, max;
   float scale;
   max = 0;
   for (i = 0; i < num; i++)
-  { if (val[i] > max)
+  {
+    if (val[i] > max)
       max = val[i];
   }
   if (height < 20)
-  { height = max;
+  {
+    height = max;
     if (height < 20)
       height = 20;
     else if (height > 256)
@@ -1324,7 +1419,8 @@ int html_hist(struct soap *soap, const char *title, size_t barwidth, size_t heig
   if (soap_send(soap, buf))
     return soap->error;
   for (i = 0; i < num; i++)
-  { unsigned long bar = (scale * val[i] + 0.5);
+  {
+    unsigned long bar = (scale * val[i] + 0.5);
     if (bar >= 1)
       (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "\
 <td bgcolor='#FFFFFF'><a onmouseover=\"window.status='%lu';return true\" onmouseout=\"window.status='';return true\" href='#%s'><img src='top.gif' alt='' width='%lu' height='1' align='bottom' border='0'><br><img src='bar.gif' alt='' width='%lu' height='%lu' align='bottom' border='0'></a></td>", (unsigned long)i, title && strlen(title) < 80 ? title : "", (unsigned long)barwidth, (unsigned long)barwidth, bar - 1);
@@ -1344,7 +1440,8 @@ int html_hist(struct soap *soap, const char *title, size_t barwidth, size_t heig
   if (soap_send(soap, buf))
     return soap->error;
   for (i = 0; i < num; i++)
-  { (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "<td%s>%s</td>", (i == highlight) ? " bgcolor='#777777'" : "", key ? key[i] : "<img src='bar.gif'>");
+  {
+    (SOAP_SNPRINTF(buf, sizeof(buf), 4095), "<td%s>%s</td>", (i == highlight) ? " bgcolor='#777777'" : "", key ? key[i] : "<img src='bar.gif'>");
     if (soap_send(soap, buf))
       return soap->error;
   }
@@ -1372,20 +1469,20 @@ int html_hist(struct soap *soap, const char *title, size_t barwidth, size_t heig
  *
 \******************************************************************************/
 
-#ifdef WITH_OPENSSL
+#if defined(WITH_OPENSSL)
 
 struct CRYPTO_dynlock_value
 {
   MUTEX_TYPE mutex;
 };
 
-static MUTEX_TYPE *mutex_buf;
+static MUTEX_TYPE *mutex_buf = NULL;
 
 static struct CRYPTO_dynlock_value *dyn_create_function(const char *file, int line)
 {
   struct CRYPTO_dynlock_value *value;
   (void)file; (void)line;
-  value = (struct CRYPTO_dynlock_value*)malloc(sizeof(struct CRYPTO_dynlock_value));
+  value = (struct CRYPTO_dynlock_value*)OPENSSL_malloc(sizeof(struct CRYPTO_dynlock_value));
   if (value)
     MUTEX_SETUP(value->mutex);
   return value;
@@ -1404,7 +1501,7 @@ static void dyn_destroy_function(struct CRYPTO_dynlock_value *l, const char *fil
 {
   (void)file; (void)line;
   MUTEX_CLEANUP(l->mutex);
-  free(l);
+  OPENSSL_free(l);
 }
 
 static void locking_function(int mode, int n, const char *file, int line)
@@ -1424,7 +1521,7 @@ static unsigned long id_function()
 int CRYPTO_thread_setup()
 {
   int i;
-  mutex_buf = (MUTEX_TYPE*)malloc(CRYPTO_num_locks() * sizeof(MUTEX_TYPE));
+  mutex_buf = (MUTEX_TYPE*)OPENSSL_malloc(CRYPTO_num_locks() * sizeof(MUTEX_TYPE));
   if (!mutex_buf)
     return SOAP_EOM;
   for (i = 0; i < CRYPTO_num_locks(); i++)
@@ -1449,27 +1546,20 @@ void CRYPTO_thread_cleanup()
   CRYPTO_set_dynlock_destroy_callback(NULL);
   for (i = 0; i < CRYPTO_num_locks(); i++)
     MUTEX_CLEANUP(mutex_buf[i]);
-  free(mutex_buf);
+  OPENSSL_free(mutex_buf);
   mutex_buf = NULL;
 }
 
 #else
 
-/* OpenSSL not used */
+/* OpenSSL not used or OpenSSL prior to 1.1.0 */
 
 int CRYPTO_thread_setup()
-{ return SOAP_OK;
+{
+  return SOAP_OK;
 }
 
 void CRYPTO_thread_cleanup()
 { }
 
 #endif
-
-/******************************************************************************\
- *
- *      SIGPIPE
- *
-\******************************************************************************/
-
-void sigpipe_handle(int x) { }
