@@ -39,7 +39,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "wsdlH.h"
 
 class Message
-{ public:
+{
+  public:
     wsdl__message *message;
     const char *name;
     const char *URI;
@@ -47,7 +48,7 @@ class Message
     soap__useChoice use;
     const char *encodingStyle;
     const char *action;
-    xs__element *element;
+    const xs__element *element;
     const char *body_parts;
     wsdl__part *part;
     bool mustUnderstand;
@@ -59,22 +60,24 @@ class Message
     const char *documentation;
     const char *ext_documentation;
     vector<const wsp__Policy*> policy;
-    void generate(Types&, const char *sep, bool anonymous, bool remark, bool response, bool optional);
+    void generate(Types&, const char *sep, bool anonymous, bool remark, bool response, bool optional, bool rest);
 };
 
 typedef map<const char*, Message*, ltstr> MapOfStringToMessage;
 
 class Operation
-{ public:
+{
+  public:
     const wsdl__operation *operation;
     const char *prefix;
     const char *URI;
     const char *name;
     const char *mep;			// WSDL 2.0
+    bool is_rest;
     const char *protocol;
     soap__styleChoice style;
     const char *parameterOrder;
-    const char *action;			// SOAP action or HTTP location
+    const char *action;			// SOAP action, REST HTTP location, or REST path /path
     const char *input_name;
     const char *output_name;
     Message *input; 			// name, use, and parts
@@ -88,7 +91,8 @@ class Operation
 };
 
 class Service
-{ public:
+{
+  public:
     const char *prefix;			// a gSOAP service has a unique namespace
     const char *URI;
     const char *name;			// binding name
@@ -114,7 +118,8 @@ class Service
 typedef map<const char*, Service*, ltstr> MapOfStringToService;
 
 class Definitions
-{ public:
+{
+  public:
     Types types;				// to process schema type information
     MapOfStringToService services;		// service information gathered
     bool soap12;
@@ -127,6 +132,8 @@ class Definitions
     void analyze_headers(const wsdl__definitions&, Service*, wsdl__ext_ioput*, wsdl__ext_ioput*);
     void analyze_faults(const wsdl__definitions&, Service*, Operation*, vector<wsdl__ext_operation>::const_iterator&);
     Message *analyze_fault(const wsdl__definitions&, Service*, const wsdl__ext_fault&);
+    void analyze_application(const wsdl__definitions&);
+    void analyze_resource(const wsdl__definitions&, Service*, const wadl__resource_USCOREtype*, const char*, const char*, const char*);
     void generate();
 };
 
