@@ -47,7 +47,8 @@ typedef map<const char*, CType, ltstr> MapOfStringToCType;
 enum Lookup { NOLOOKUP, LOOKUP };
 
 class Types
-{ public:
+{
+  public:
     SetOfString		knames;	// keywords, reserved words, class names, and typedefs
     MapOfStringToString modtypemap;
     MapOfStringToString deftypemap;
@@ -55,6 +56,7 @@ class Types
     MapOfStringToString ptrtypemap;
     MapOfStringToString smptypemap;
     MapOfStringToCType	ctypemap;
+    MapOfStringToString wnames;	// name -> wrapper name
     MapOfPairToString	qnames;	// (URI,name) -> name
     MapOfStringToString	uris;	// URI -> prefix
     MapOfStringToNum	syms;	// prefix -> count (ns1, ns2, ...)
@@ -67,7 +69,8 @@ class Types
     int                 gnum;   // enum name index, TODO: consider map of URI to count per URI
     bool                with_union;
     bool                fake_union;
-    size_t              omitted; // number of -O2 removed types
+    size_t              total; // total number of types
+    size_t              omitted; // number of -O removed types
     Types();
     void init();
     int read(const char *file);
@@ -75,7 +78,7 @@ class Types
     const char *fname(const char *prefix, const char *URI, const char *qname, SetOfString *reserved, enum Lookup lookup, bool isqname);
   public:
     const char *aname(const char *prefix, const char *URI, const char *qname, SetOfString *reserved = NULL);
-    const char *wname(const char *prefix, const char *URI, const char *qname);
+    const char *wname(const char *prefix, const char *URI, const char *qname, enum Lookup lookup);
     const char *cname(const char *prefix, const char *URI, const char *qname);
     const char *tname(const char *prefix, const char *URI, const char *qname);
     const char *tnameptr(bool, const char *prefix, const char *URI, const char *qname);
