@@ -59,7 +59,7 @@ int is_builtin_qname(const char *QName)
     if (*QName == '#') // reserved QNames
       return 0;
     if (*QName != '"')
-      return 1;	// if the QName does not start with a ", it must be in the nsmap
+      return 1; // if the QName does not start with a ", it must be in the nsmap
     const char *s = strchr(QName + 1, '"');
     if (s)
     {
@@ -72,9 +72,18 @@ int is_builtin_qname(const char *QName)
   return 0;
 }
 
+xsd__QName make_qname(xs__schema& schema, const char *name)
+{
+  const char *URI = schema.targetNamespace ? schema.targetNamespace : "";
+  size_t n = strlen(URI) + strlen(name) + 3;
+  char *s = (char*)soap_malloc(schema.soap, n + 1);
+  (SOAP_SNPRINTF(s, n + 1, n), "\"%s\":%s", URI, name);
+  return s;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //
-//	wsdl
+//      wsdl
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -90,7 +99,7 @@ wsdl__definitions::wsdl__definitions()
   soap_register_plugin(soap, http_da);
 #endif
 #ifdef WITH_OPENSSL
-  soap_ssl_client_context(soap, SOAP_SSL_NO_AUTHENTICATION, NULL, NULL, NULL, NULL, NULL);
+  soap_ssl_client_context(soap, SOAP_SSL_NO_AUTHENTICATION | SOAP_SSLv3_TLSv1, NULL, NULL, NULL, NULL, NULL);
 #endif
   soap_set_namespaces(soap, namespaces);
   soap_default(soap);
@@ -110,6 +119,8 @@ wsdl__definitions::wsdl__definitions()
   updated = false;
   location = NULL;
   redirs = 0;
+  appRef = NULL;
+  used = false;
 }
 
 wsdl__definitions::wsdl__definitions(struct soap *copy)
@@ -125,6 +136,7 @@ wsdl__definitions::wsdl__definitions(struct soap *copy)
   updated = false;
   location = NULL;
   redirs = 0;
+  appRef = NULL;
 }
 
 wsdl__definitions::wsdl__definitions(struct soap *copy, const char *cwd, const char *loc)
@@ -140,6 +152,7 @@ wsdl__definitions::wsdl__definitions(struct soap *copy, const char *cwd, const c
   updated = false;
   location = NULL;
   redirs = 0;
+  appRef = NULL;
   read(cwd, loc);
 }
 
@@ -181,7 +194,7 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
   if (!cwd)
     cwd = cwd_path;
   if (vflag)
-    fprintf(stderr, "\nOpening WSDL/XSD '%s' from '%s'\n", loc ? loc : "(stdin)", cwd ? cwd : "./");
+    fprintf(stderr, "\nOpening WSDL/WADL or XSD '%s' relative to '%s'\n", loc ? loc : "(stdin)", cwd ? cwd : "./");
   if (loc)
   {
     if (soap->recvfd > 2)
@@ -200,13 +213,17 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
 #else
     if (!strncmp(loc, "https://", 8))
     {
-      fprintf(stderr, "\nCannot connect to https site: no SSL support, please rebuild wsdl2h with SSL or download the files and rerun wsdl2h\n");
+#ifdef WIN32
+      fprintf(stderr, "\nCannot connect to https site: SSL/TLS support not enabled in this version. Visit https://www.genivia.com/downloads.html to download the secure version of wsdl2h.exe that supports SSL/TLS to connect to https sites.\n");
+#else
+      fprintf(stderr, "\nCannot connect to https site: SSL/TLS support not enabled, please rebuild wsdl2h with SSL/TLS enabled using 'make secure' or download the WSDL/WADL and XSD files and rerun wsdl2h on these files directly by specifying the file names on the command line.\n");
+#endif
       exit(1);
     }
     else if (!strncmp(loc, "http://", 7))
 #endif
     {
-      fprintf(stderr, "%*sConnecting to '%s' to retrieve WSDL/XSD...", 2*openfiles, "", loc);
+      fprintf(stderr, "%*sConnecting to '%s' to retrieve WSDL/WADL or XSD...", 2*openfiles, "", loc);
       location = soap_strdup(soap, loc);
       if (soap_connect_command(soap, SOAP_GET, location, NULL))
       {
@@ -232,7 +249,8 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
       fprintf(stderr, "%*sConnecting to '%s' to retrieve '%s'...", 2*openfiles, "", location, loc);
       if (soap_connect_command(soap, SOAP_GET, location, NULL))
       {
-        fprintf(stderr, "\n\nError: connection failed\n");
+        fprintf(stderr, "\n\nError: failed to retrieve '%s'\n", loc);
+        soap_print_fault(soap, stderr);
         exit(1);
       }
       fprintf(stderr, " connected, receiving...\n");
@@ -240,12 +258,14 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
     }
     else
     {
+      if (!strncmp(loc, "file://", 7))
+        loc += 7;
       soap->recvfd = open(loc, O_RDONLY, 0);
       if (soap->recvfd < 0)
       {
         if (cwd)
         {
-	  size_t l = strlen(cwd) + strlen(loc);
+          size_t l = strlen(cwd) + strlen(loc);
           location = (char*)soap_malloc(soap, l + 2);
           soap_strcpy(location, l + 2, cwd);
           char *s = strrchr(location, '/');
@@ -254,27 +274,33 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
             s = strrchr(location, '\\');
 #endif
           if (s)
+          {
             *s = '\0';
-	  size_t n = strlen(location);
-          soap_strcpy(location + n, l + 2 - n, "/");
-	  ++n;
-          soap_strcpy(location + n, l + 2 - n, loc);
-          if (!strncmp(location, "file://", 7))
-            location += 7;
-          soap->recvfd = open(location, O_RDONLY, 0);
+            size_t n = strlen(location);
+            soap_strcpy(location + n, l + 2 - n, "/");
+            ++n;
+            soap_strcpy(location + n, l + 2 - n, loc);
+            if (!strncmp(location, "file://", 7))
+              location += 7;
+            soap->recvfd = open(location, O_RDONLY, 0);
+            if (vflag)
+              cerr << "Opening file " << location << (soap->recvfd < 0 ? " failed" : " successful") << endl;
+          }
         }
         if (soap->recvfd < 0 && import_path)
         {
-	  size_t l = strlen(import_path) + strlen(loc);
+          size_t l = strlen(import_path) + strlen(loc);
           location = (char*)soap_malloc(soap, l + 2);
           soap_strcpy(location, l + 2, import_path);
-	  size_t n = strlen(location);
+          size_t n = strlen(location);
           soap_strcpy(location + n, l + 2 - n, "/");
-	  ++n;
+          ++n;
           soap_strcpy(location + n, l + 2 - n, loc);
           if (!strncmp(location, "file://", 7))
             location += 7;
           soap->recvfd = open(location, O_RDONLY, 0);
+          if (vflag)
+            cerr << "Opening file " << location << (soap->recvfd < 0 ? " failed" : " successful") << endl;
         }
         if (soap->recvfd < 0)
         {
@@ -283,7 +309,9 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
         }
       }
       else
+      {
         location = soap_strdup(soap, loc);
+      }
       fprintf(stderr, "%*sReading '%s'...\n", 2*openfiles, "", location);
       openfiles++;
     }
@@ -296,27 +324,48 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
   {
     // handle sloppy WSDLs that import schemas at the top level rather than
     // importing them in <types>
+    // handle WADL when used in place of WSDL
     if (soap->error == SOAP_TAG_MISMATCH && soap->level == 0)
     {
       soap_retry(soap);
       xs__schema *schema = soap_new_xs__schema(soap);
+      schema->sourceLocation(location);
       schema->soap_in(soap, "xs:schema", NULL);
-      if (soap->error)
+      if (soap->error == SOAP_TAG_MISMATCH && soap->level == 0)
       {
-        fprintf(stderr, "\nAn error occurred while parsing WSDL or XSD from '%s'\n", loc ? loc : "(stdin)");
-        soap_print_fault(soap, stderr);
-        if (soap->error < 200)
-          soap_print_fault_location(soap, stderr);
-        exit(1);
+        soap_retry(soap);
+        wadl__application *app = soap_new_wadl__application(soap);
+        app->soap_in(soap, "wadl:application", NULL);
+        if (soap->error)
+        {
+          fprintf(stderr, "\nAn error occurred while parsing WSDL/WADL or XSD from '%s'\n", loc ? loc : "(stdin)");
+          soap_print_fault(soap, stderr);
+          if (soap->error < 200)
+            soap_print_fault_location(soap, stderr);
+          exit(1);
+        }
+        appRef = app;
+        preprocess();
+      }
+      else
+      {
+        if (soap->error)
+        {
+          fprintf(stderr, "\nAn error occurred while parsing WSDL/WADL or XSD from '%s'\n", loc ? loc : "(stdin)");
+          soap_print_fault(soap, stderr);
+          if (soap->error < 200)
+            soap_print_fault_location(soap, stderr);
+          exit(1);
+        }
+        name = NULL;
+        targetNamespace = schema->targetNamespace;
+        if (vflag)
+          cerr << "Found schema '" << (targetNamespace ? targetNamespace : "(null)") << "' when expecting WSDL" << endl;
+        types = soap_new_wsdl__types(soap);
+        types->documentation = NULL;
+        types->xs__schema_.push_back(schema);
+        types->preprocess(*this);
       }
-      name = NULL;
-      targetNamespace = schema->targetNamespace;
-      if (vflag)
-        cerr << "Found schema '" << (targetNamespace ? targetNamespace : "(null)") << "' when expecting WSDL" << endl;
-      types = soap_new_wsdl__types(soap);
-      types->documentation = NULL;
-      types->xs__schema_.push_back(schema);
-      types->preprocess(*this);
     }
     // check HTTP redirect (socket was closed)
     else if ((soap->error >= 301 && soap->error <= 303) || soap->error == 307)
@@ -350,12 +399,14 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
         redirs--;
       }
       else
+      {
         fprintf(stderr, "Authentication failed, use option -r:uid:pwd and (re)build with OpenSSL to enable digest authentication\n");
+      }
       return r;
     }
     else
     {
-      fprintf(stderr, "\nAn error occurred while parsing WSDL from '%s'\n", loc ? loc : "(stdin)");
+      fprintf(stderr, "\nAn error occurred while parsing WSDL/WADL and XSD from '%s'\n", loc ? loc : "(stdin)");
       soap_print_fault(soap, stderr);
       if (soap->error < 200)
         soap_print_fault_location(soap, stderr);
@@ -371,7 +422,9 @@ int wsdl__definitions::read(const char *cwd, const char *loc)
     soap->recvfd = -1;
   }
   else
+  {
     soap_closesock(soap);
+  }
   cwd_path = cwd_temp;
   return SOAP_OK;
 }
@@ -380,6 +433,9 @@ int wsdl__definitions::preprocess()
 {
   if (vflag)
     cerr << "Preprocessing wsdl definitions '" << (location ? location : "(null)") << "' namespace '" << (targetNamespace ? targetNamespace : "(null)") << "'" << endl;
+  // process WADL app
+  if (appPtr())
+    appPtr()->preprocess(*this);
   // process import
   for (vector<wsdl__import>::iterator im1 = import.begin(); im1 != import.end(); ++im1)
     (*im1).preprocess(*this);
@@ -488,6 +544,9 @@ int wsdl__definitions::traverse()
   // then process the types
   if (types)
     types->traverse(*this);
+  // process WADL
+  if (appRef)
+    appRef->traverse(*this);
   // process messages before portType
   for (vector<wsdl__message>::iterator mg = message.begin(); mg != message.end(); ++mg)
     (*mg).traverse(*this);
@@ -521,6 +580,34 @@ const char *wsdl__definitions::sourceLocation()
   return location;
 }
 
+char *wsdl__definitions::absoluteLocation(const char *loc) const
+{
+  if (!location)
+    return soap_strdup(soap, loc);
+  if (!strncmp(loc, "http://", 7) || !strncmp(loc, "https://", 8))
+    return soap_strdup(soap, loc);
+  if (!strncmp(loc, "file://", 7))
+    loc += 7;
+  const char *s = strrchr(location, '/');
+  if (!s)
+    return soap_strdup(soap, loc);
+  if (strncmp(loc, "../", 3))
+    return soap_strdup(soap, loc);
+  while (!strncmp(loc, "../", 3) && s > location)
+  {
+    while (--s >= location)
+      if (*s == '/')
+        break;
+    loc += 3;
+  }
+  size_t n = s - location + 1;
+  size_t l = n + strlen(loc);
+  char *abs = (char*)soap_malloc(soap, l + 1);
+  soap_strncpy(abs, l + 1, location, n);
+  soap_strcpy(abs + n, l + 1 - n, loc);
+  return abs;
+}
+
 int wsdl__definitions::error()
 {
   return soap->error;
@@ -543,12 +630,23 @@ void wsdl__definitions::builtinType(const char *type)
   builtinTypeSet.insert(type);
 }
 
+void wsdl__definitions::builtinTypeDerivation(xs__schema& schema, const char *base, const char *derived)
+{
+  builtinTypeMap[make_qname(schema, derived)] = base;
+}
+
 void wsdl__definitions::builtinTypes(const SetOfString& types)
 {
   for (SetOfString::const_iterator tp = types.begin(); tp != types.end(); ++tp)
     builtinTypeSet.insert(*tp);
 }
 
+void wsdl__definitions::builtinTypeDerivations(const MapOfStringToString& derivations)
+{
+  for (MapOfStringToString::const_iterator dp = derivations.begin(); dp != derivations.end(); ++dp)
+    builtinTypeMap.insert(*dp);
+}
+
 void wsdl__definitions::builtinElement(const char *element)
 {
   builtinElementSet.insert(element);
@@ -576,6 +674,11 @@ const SetOfString& wsdl__definitions::builtinTypes() const
   return builtinTypeSet;
 }
 
+const MapOfStringToString& wsdl__definitions::builtinTypeDerivations() const
+{
+  return builtinTypeMap;
+}
+
 const SetOfString& wsdl__definitions::builtinElements() const
 {
   return builtinElementSet;
@@ -586,6 +689,36 @@ const SetOfString& wsdl__definitions::builtinAttributes() const
   return builtinAttributeSet;
 }
 
+void wsdl__definitions::appPtr(wadl__application *app)
+{
+  appRef = app;
+}
+
+wadl__application *wsdl__definitions::appPtr() const
+{
+  return appRef;
+}
+
+void wsdl__definitions::mark()
+{
+  if (Oflag > 1 && !used)
+  {
+    used = true;
+    for (vector<wsdl__import>::iterator im = import.begin(); im != import.end(); ++im)
+      (*im).mark();
+    if (types)
+      types->mark();
+    if (appRef)
+      appRef->mark();
+    for (vector<wsdl__portType>::iterator pt = interface_.begin(); pt != interface_.end(); ++pt)
+      (*pt).mark();
+    for (vector<wsdl__message>::iterator mg = message.begin(); mg != message.end(); ++mg)
+      (*mg).mark();
+    for (vector<wsdl__binding>::iterator bg = binding.begin(); bg != binding.end(); ++bg)
+      (*bg).mark();
+  }
+}
+
 int wsdl__service::traverse(wsdl__definitions& definitions)
 {
   if (vflag)
@@ -784,6 +917,13 @@ wsdl__portType *wsdl__binding::portTypePtr() const
   return portTypeRef;
 }
 
+void wsdl__binding::mark()
+{
+  if (Oflag > 1)
+    for (vector<wsdl__ext_operation>::iterator i = operation.begin(); i != operation.end(); ++i)
+      (*i).mark();
+}
+
 wsdl__ext_operation::wsdl__ext_operation()
 {
   operationRef = NULL;
@@ -933,6 +1073,17 @@ wsdl__operation *wsdl__ext_operation::operationPtr() const
   return operationRef;
 }
 
+void wsdl__ext_operation::mark()
+{
+  if (Oflag > 1)
+  {
+    if (input)
+      input->mark();
+    if (output)
+      output->mark();
+  }
+}
+
 int wsdl__ext_ioput::traverse(wsdl__definitions& definitions)
 {
   if (vflag)
@@ -950,6 +1101,13 @@ int wsdl__ext_ioput::traverse(wsdl__definitions& definitions)
   return SOAP_OK;
 }
 
+void wsdl__ext_ioput::mark()
+{
+  if (Oflag > 1)
+    for (std::vector<wsoap__header>::iterator sh = wsoap__header_.begin(); sh != wsoap__header_.end(); ++sh)
+      (*sh).mark();
+}
+
 wsdl__ext_fault::wsdl__ext_fault()
 {
   faultRef = NULL;
@@ -1031,6 +1189,17 @@ wsdl__definitions *wsdl__portType::definitionsPtr() const
   return definitionsRef;
 }
 
+void wsdl__portType::mark()
+{
+  if (Oflag > 1)
+  {
+    for (std::vector<wsdl__fault>::iterator ft = fault.begin(); ft != fault.end(); ++ft)
+      (*ft).mark();
+    for (std::vector<wsdl__operation>::iterator op = operation.begin(); op != operation.end(); ++op)
+      (*op).mark();
+  }
+}
+
 int wsdl__operation::traverse(wsdl__definitions& definitions)
 {
   if (vflag)
@@ -1061,6 +1230,31 @@ int wsdl__operation::traverse(wsdl__definitions& definitions)
   return SOAP_OK;
 }
 
+void wsdl__operation::mark()
+{
+  if (Oflag > 1)
+  {
+    if (__union1 == SOAP_UNION_wsdl__union_ioput_input)
+      if (__ioput1.input)
+        __ioput1.input->mark();
+    if (__union1 == SOAP_UNION_wsdl__union_ioput_output)
+      if (__ioput1.output)
+        __ioput1.output->mark();
+    if (__union2 == SOAP_UNION_wsdl__union_ioput_input)
+      if (__ioput2.input)
+        __ioput2.input->mark();
+    if (__union2 == SOAP_UNION_wsdl__union_ioput_output)
+      if (__ioput2.output)
+        __ioput2.output->mark();
+    for (vector<wsdl__fault>::iterator i = fault.begin(); i != fault.end(); ++i)
+      (*i).mark();
+    for (vector<wsdl__fault>::iterator fi = infault.begin(); fi != infault.end(); ++fi)
+      (*fi).mark();
+    for (vector<wsdl__fault>::iterator fo = outfault.begin(); fo != outfault.end(); ++fo)
+      (*fo).mark();
+  }
+}
+
 wsdl__ioput::wsdl__ioput()
 {
   messageRef = NULL;
@@ -1135,7 +1329,7 @@ int wsdl__ioput::traverse(wsdl__definitions& definitions)
             {
               if ((*message).name && !strcmp((*message).name, token))
               {
-        	messageRef = &(*message);
+                messageRef = &(*message);
                 if (vflag)
                   cerr << "    Found input/output '" << (name ? name : "(null)") << "' message '" << (token ? token : "(null)") << "'" << endl;
                 break;
@@ -1176,6 +1370,13 @@ xs__element *wsdl__ioput::elementPtr() const
   return elementRef;
 }
 
+void wsdl__ioput::mark()
+{
+  if (Oflag > 1)
+    if (elementPtr())
+      elementPtr()->mark();
+}
+
 wsdl__fault::wsdl__fault()
 {
   messageRef = NULL;
@@ -1222,9 +1423,9 @@ int wsdl__fault::traverse(wsdl__definitions& definitions)
             {
               for (vector<wsdl__fault>::iterator fault = (*i).fault.begin(); fault != (*i).fault.end(); ++fault)
               {
-        	if ((*fault).name && !strcmp((*fault).name, token))
+                if ((*fault).name && !strcmp((*fault).name, token))
                 {
-        	  elementRef = (*fault).elementPtr();
+                  elementRef = (*fault).elementPtr();
                   if (vflag)
                     cerr << "   Found fault '" << (ref ? ref : "(null)") << "' element '" << (token ? token : "(null)") << "'" << endl;
                   break;
@@ -1305,7 +1506,7 @@ int wsdl__fault::traverse(wsdl__definitions& definitions)
             {
               if ((*message).name && !strcmp((*message).name, token))
               {
-        	messageRef = &(*message);
+                messageRef = &(*message);
                 if (vflag)
                   cerr << "    Found operation fault '" << (name ? name : "(null)") << "' message '" << (token ? token : "(null)") << "'" << endl;
                 break;
@@ -1346,6 +1547,13 @@ xs__element *wsdl__fault::elementPtr() const
   return elementRef;
 }
 
+void wsdl__fault::mark()
+{
+  if (Oflag > 1)
+    if (elementPtr())
+      elementPtr()->mark();
+}
+
 int wsdl__message::traverse(wsdl__definitions& definitions)
 {
   if (vflag)
@@ -1359,11 +1567,22 @@ int wsdl__message::traverse(wsdl__definitions& definitions)
   return SOAP_OK;
 }
 
+void wsdl__message::mark()
+{
+  if (Oflag > 1)
+    for (vector<wsdl__part>::iterator i = part.begin(); i != part.end(); ++i)
+      (*i).mark();
+}
+
 wsdl__part::wsdl__part()
 {
   elementRef = NULL;
   simpleTypeRef = NULL;
   complexTypeRef = NULL;
+  optional = false;
+  repeating = false;
+  fixed = false;
+  default_ = NULL;
 }
 
 int wsdl__part::traverse(wsdl__definitions& definitions)
@@ -1482,6 +1701,70 @@ xs__complexType *wsdl__part::complexTypePtr() const
   return complexTypeRef;
 }
 
+void wsdl__part::set_optional(bool b)
+{
+  optional = b;
+}
+
+void wsdl__part::set_fixed(bool b)
+{
+  fixed = b;
+}
+
+void wsdl__part::set_repeating(bool b)
+{
+  repeating = b;
+}
+
+void wsdl__part::set_default(char* s)
+{
+  default_ = s;
+}
+
+void wsdl__part::set_option(char* s)
+{
+  if (s)
+    option.push_back(s);
+}
+
+bool wsdl__part::is_optional() const
+{
+  return optional;
+}
+
+bool wsdl__part::is_fixed() const
+{
+  return fixed;
+}
+
+bool wsdl__part::is_repeating() const
+{
+  return repeating;
+}
+
+const char *wsdl__part::get_default() const
+{
+  return default_;
+}
+
+const std::vector<char*>& wsdl__part::options() const
+{
+  return option;
+}
+
+void wsdl__part::mark()
+{
+  if (Oflag > 1)
+  {
+    if (elementPtr())
+      elementPtr()->mark();
+    if (simpleTypePtr())
+      simpleTypePtr()->mark();
+    if (complexTypePtr())
+      complexTypePtr()->mark();
+  }
+}
+
 int wsdl__types::preprocess(wsdl__definitions& definitions)
 {
   if (vflag)
@@ -1491,6 +1774,8 @@ int wsdl__types::preprocess(wsdl__definitions& definitions)
     targetNamespace = definitions.targetNamespace;
     xs__schema_.push_back(this);
   }
+  for (vector<xs__schema*>::iterator schema = xs__schema_.begin(); schema != xs__schema_.end(); ++schema)
+    (*schema)->sourceLocation(definitions.sourceLocation());
 again:
   // link imported schemas, need to repeat when <types> is extended with new imported schema (from inside another schema, etc.)
   for (vector<xs__schema*>::iterator schema1 = xs__schema_.begin(); schema1 != xs__schema_.end(); ++schema1)
@@ -1516,36 +1801,43 @@ again:
     for (vector<xs__import>::iterator import = (*schema2)->import.begin(); import != (*schema2)->import.end(); ++import)
     {
       bool found = false;
-      if ((*import).schemaPtr())
-        found = true;
       if (vflag)
         cerr << "Preprocessing schema '" << (*schema2)->targetNamespace << "' import '" << ((*import).namespace_ ? (*import).namespace_ : "(null)") << "'" << endl; 
       if (!found && (*import).namespace_)
       {
-	for (SetOfString::const_iterator i = exturis.begin(); i != exturis.end(); ++i)
-	{
-	  if (!soap_tag_cmp((*import).namespace_, *i))
-	  {
-	    found = true;
-	    break;
-	  }
-	}
+        for (SetOfString::const_iterator i = exturis.begin(); i != exturis.end(); ++i)
+        {
+          if (!soap_tag_cmp((*import).namespace_, *i))
+          {
+            found = true;
+            break;
+          }
+        }
       }
       if (!found && !iflag) // don't import any of the schemas in the .nsmap table (or when -i option is used)
       {
-        xs__schema *importschema;
-        importschema = (*import).schemaPtr();
+        xs__schema *importschema = (*import).schemaPtr();
+        if (importschema)
+        {
+#if 0 // no longer applicable as imports are preprocessed
         if (!importschema)
         {
           const char *s = (*import).schemaLocation;
-          if (!s)
+          if (!s && (*import).location)
+          {
+            s = (*import).schemaLocation = (*import).location; // work around for Microsoft bugs
+            cerr << "Schema import with namespace '" << ((*import).namespace_ ? (*import).namespace_ : "(null)") << "' has a 'location' attribute specified but a 'schemaLocation' attribute must be used, please inform the author of this WSDL to correct this problem" << endl;
+          }
+          else if (!s)
+          {
             s = (*import).namespace_;
+          }
           if (!s)
             continue;
-          importschema = new xs__schema(definitions.soap, (*schema2)->sourceLocation(), s);
+          importschema = new xs__schema(definitions.soap, (*schema2)->sourceLocation(), (*schema2)->absoluteLocation(s));
           if (!(*import).namespace_)
           {
-	    if (importschema->targetNamespace)
+            if (importschema->targetNamespace)
               (*import).namespace_ = importschema->targetNamespace;
             else
               (*import).namespace_ = soap_strdup(definitions.soap, "");
@@ -1558,25 +1850,52 @@ again:
           {
             if ((*schema3)->targetNamespace && !strcmp((*import).namespace_, (*schema3)->targetNamespace))
             {
-	      (*import).schemaPtr(*schema3);
-	      if ((*schema3) == this || // WSDL 2.0 <types> has no FormDefaults
-		  (*schema3)->empty())  // schema w/o components, only imports
-	      {
-		(*schema3)->elementFormDefault = importschema->elementFormDefault;
-		(*schema3)->attributeFormDefault = importschema->attributeFormDefault;
-	      }
-	      (*schema3)->insert(*importschema); // merge content
-	      goto again;
+              (*import).schemaPtr(*schema3);
+              if ((*schema3) == this || // WSDL 2.0 <types> has no FormDefaults
+                  (*schema3)->empty())  // schema w/o components, only imports
+              {
+                (*schema3)->elementFormDefault = importschema->elementFormDefault;
+                (*schema3)->attributeFormDefault = importschema->attributeFormDefault;
+              }
+              (*schema3)->insert(*importschema); // merge content
+              goto again;
             }
           }
         }
-        if (importschema)
+#endif
+          if (!found)
+          {
+            for (vector<xs__schema*>::const_iterator schema3 = xs__schema_.begin(); schema3 != xs__schema_.end(); ++schema3)
+            {
+              if (*schema3 == importschema)
+              {
+                found = true;
+              }
+              else if ((*schema3)->targetNamespace && (*import).namespace_ && !strcmp((*import).namespace_, (*schema3)->targetNamespace))
+              {
+                (*import).schemaPtr(*schema3);
+                if ((*schema3) == this || // WSDL 2.0 <types> has no FormDefaults
+                    (*schema3)->empty())  // schema w/o components, only imports
+                {
+                  (*schema3)->elementFormDefault = importschema->elementFormDefault;
+                  (*schema3)->attributeFormDefault = importschema->attributeFormDefault;
+                }
+                (*schema3)->insert(*importschema); // merge content
+                goto again;
+              }
+            }
+          }
+          if (!found)
+          {
+            xs__schema_.push_back(importschema);
+            if (vflag)
+              cerr << "Adding schema '" << importschema->targetNamespace << "'" << endl;
+            goto again;
+          }
+        }
+        else if (!Wflag)
         {
-          (*import).schemaPtr(importschema);
-          xs__schema_.push_back(importschema);
-          if (vflag)
-            cerr << "Adding schema '" << importschema->targetNamespace << "'" << endl;
-          goto again;
+          fprintf(stderr, "\nWarning: no schemaLocation in <import namespace=\"%s\"> to load schema: unknown schema\n", (*import).namespace_ ? (*import).namespace_ : "(null)");
         }
       }
     }
@@ -1587,15 +1906,15 @@ again:
     {
       cerr << endl << "Schema " << ((*schema4)->targetNamespace ? (*schema4)->targetNamespace : "") << " " << ((*schema4)->sourceLocation() ? (*schema4)->sourceLocation() : "") << endl;
       for (vector<xs__import>::iterator im = (*schema4)->import.begin(); im != (*schema4)->import.end(); ++im)
-	cerr << "  import " << ((*im).namespace_ ? (*im).namespace_ : "") << " " << ((*im).schemaLocation ? (*im).schemaLocation : "") << endl;
+        cerr << "  import " << ((*im).namespace_ ? (*im).namespace_ : "") << " " << ((*im).schemaLocation ? (*im).schemaLocation : "") << endl;
       for (vector<xs__simpleType>::iterator st = (*schema4)->simpleType.begin(); st != (*schema4)->simpleType.end(); ++st)
-	cerr << "  simpleType " << ((*st).name ? (*st).name : "") << endl;
+        cerr << "  simpleType " << ((*st).name ? (*st).name : "") << endl;
       for (vector<xs__complexType>::iterator ct = (*schema4)->complexType.begin(); ct != (*schema4)->complexType.end(); ++ct)
-	cerr << "  complexType " << ((*ct).name ? (*ct).name : "") << endl;
+        cerr << "  complexType " << ((*ct).name ? (*ct).name : "") << endl;
       for (vector<xs__element>::iterator el = (*schema4)->element.begin(); el != (*schema4)->element.end(); ++el)
-	cerr << "  element " << ((*el).name ? (*el).name : "") << endl;
+        cerr << "  element " << ((*el).name ? (*el).name : "") << endl;
       for (vector<xs__attribute>::iterator at = (*schema4)->attribute.begin(); at != (*schema4)->attribute.end(); ++at)
-	cerr << "  attribute " << ((*at).name ? (*at).name : "") << endl;
+        cerr << "  attribute " << ((*at).name ? (*at).name : "") << endl;
     }
   }
   return SOAP_OK;
@@ -1649,16 +1968,28 @@ int wsdl__types::traverse(wsdl__definitions& definitions)
       for (SetOfString::const_iterator i = (*schema5)->builtinTypes().begin(); i != (*schema5)->builtinTypes().end(); ++i)
         cerr << " Built-in schema type '" << (*i) << "'" << endl;
     definitions.builtinTypes((*schema5)->builtinTypes());
+    definitions.builtinTypeDerivations((*schema5)->builtinTypeDerivations());
     definitions.builtinElements((*schema5)->builtinElements());
     definitions.builtinAttributes((*schema5)->builtinAttributes());
   }
   return SOAP_OK;
 }
 
+void wsdl__types::mark()
+{
+  if (Oflag > 1)
+  {
+    xs__schema::mark();
+    for (vector<xs__schema*>::iterator schema = xs__schema_.begin(); schema != xs__schema_.end(); ++schema)
+      (*schema)->mark();
+  }
+}
+
 int wsdl__import::preprocess(wsdl__definitions& definitions)
 {
   static map<const char*, wsdl__definitions*, ltstr> included;
   bool found = false;
+  location = definitions.absoluteLocation(location);
   if (vflag)
     cerr << "Preprocess wsdl import '" << (location ? location : "(null)") << "'" << endl;
   definitionsRef = NULL;
@@ -1720,7 +2051,12 @@ int wsdl__import::traverse(wsdl__definitions& definitions)
       else
         definitionsRef->targetNamespace = definitions.targetNamespace;
     }
-    return definitionsRef->traverse();
+    definitionsRef->traverse();
+    // collect imported artifacts into parent collections
+    definitions.builtinTypes(definitionsRef->builtinTypes());
+    definitions.builtinTypeDerivations(definitionsRef->builtinTypeDerivations());
+    definitions.builtinElements(definitionsRef->builtinElements());
+    definitions.builtinAttributes(definitionsRef->builtinAttributes());
   }
   return SOAP_OK;
 }
@@ -1742,9 +2078,18 @@ wsdl__import::wsdl__import()
   definitionsRef = NULL;
 }
 
+void wsdl__import::mark()
+{
+  if (Oflag > 1)
+  {
+    if (definitionsRef)
+      definitionsRef->mark();
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //
-//	streams
+//      streams
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -1797,7 +2142,7 @@ istream &operator>>(istream &i, wsdl__definitions &e)
 
 ////////////////////////////////////////////////////////////////////////////////
 //
-//	Miscellaneous
+//      Miscellaneous
 //
 ////////////////////////////////////////////////////////////////////////////////
 
