@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2000-2018, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2019, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -64,23 +64,25 @@ int _flag = 0,
     Dflag = 0,
     dflag = 0,
     eflag = 0,
+    Fflag = 0,
     fflag = 0,
     gflag = 0,
     iflag = 0,
     jflag = 0,
     kflag = 0,
-    mflag = 0,
+    Lflag = 0,
     Mflag = 0,
+    mflag = 0,
     Oflag = 0,
-    pflag = 0,
     Pflag = 0,
+    pflag = 0,
     Rflag = 0,
     sflag = 0,
-    uflag = 0,
     Uflag = 0,
+    uflag = 0,
     vflag = 0,
-    wflag = 0,
     Wflag = 0,
+    wflag = 0,
     xflag = 0,
     yflag = 0,
     zflag = 0;
@@ -114,29 +116,30 @@ const char *soap_context = "soap";
 
 const char elementformat[]             = "    %-35s  %-30s";
 const char pointerformat[]             = "    %-35s *%-30s";
-const char attributeformat[]           = "   @%-35s  %-30s";
+const char attributeformat[]           = "  @ %-35s  %-30s";
 const char templateformat[]            = "    %s<%-23s> %-30s";
 const char pointertemplateformat[]     = "    %s<%-22s> *%-30s";
 const char templateformat_open[]       = "    %s<%s";
-const char attrtemplateformat_open[]   = "   @%s<%s";
+const char attrtemplateformat_open[]   = "  @ %s<%s";
 const char arrayformat[]               = "    %-35s *__ptr%-25s";
 const char arraysizeformat[]           = "    %-35s  __size%-24s";
 const char arrayoffsetformat[]         = "//  %-35s  __offset%-22s";
-const char sizeformat[]                = "   $%-35s  __size%-24s";
-const char choiceformat[]              = "   $%-35s  __union%-23s";
+const char sizeformat[]                = "  $ %-35s  __size%-24s";
+const char choiceformat[]              = "  $ %-35s  __union%-23s";
 const char schemaformat[]              = "//gsoap %-5s schema %s:\t%s\n";
 const char serviceformat[]             = "//gsoap %-4s service %s:\t%s %s\n";
 const char paraformat[]                = "    %-35s%s%s%s";
 const char anonformat[]                = "    %-35s%s_%s%s";
-const char sizeparaformat[]            = "   $%-35s __size%s%s";
+const char sizeparaformat[]            = "  $ %-35s __size%s%s";
 const char pointertemplateparaformat[] = "    %s<%-21s>*%s%s%s";
+const char derivedformat[]             = "  [ %-35s *%-30s; ]";
 
-const char copyrightnotice[] = "\n**  The gSOAP WSDL/WADL/XSD processor for C and C++, wsdl2h release " WSDL2H_VERSION "\n**  Copyright (C) 2000-2018 Robert van Engelen, Genivia Inc.\n**  All Rights Reserved. This product is provided \"as is\", without any warranty.\n**  The wsdl2h tool and its generated software are released under the GPL.\n**  ----------------------------------------------------------------------------\n**  A commercial use license is available from Genivia Inc., contact@genivia.com\n**  ----------------------------------------------------------------------------\n\n";
+const char copyrightnotice[] = "\n**  The gSOAP WSDL/WADL/XSD processor for C and C++, wsdl2h release " WSDL2H_VERSION "\n**  Copyright (C) 2000-2019 Robert van Engelen, Genivia Inc.\n**  All Rights Reserved. This product is provided \"as is\", without any warranty.\n**  The wsdl2h tool and its generated software are released under the GPL.\n**  ----------------------------------------------------------------------------\n**  A commercial use license is available from Genivia Inc., contact@genivia.com\n**  ----------------------------------------------------------------------------\n\n";
 
 const char licensenotice[]   = "\
 --------------------------------------------------------------------------------\n\
 gSOAP XML Web services tools\n\
-Copyright (C) 2000-2018, Robert van Engelen, Genivia Inc. All Rights Reserved.\n\
+Copyright (C) 2000-2019, Robert van Engelen, Genivia Inc. All Rights Reserved.\n\
 The wsdl2h tool and its generated software are released under the GPL.\n\
 This software is released under the GPL with the additional exemption that\n\
 compiling, linking, and/or using OpenSSL is allowed.\n\
@@ -190,13 +193,13 @@ int main(int argc, char **argv)
   {
     if (def.types.omitted)
     {
-      fprintf(stderr, "\nOptimization (-O2): removed type definitions of %zu unused schema components\n", def.types.omitted);
+      fprintf(stderr, "\nOptimization (-O%d): removes %zu definitions of unused schema components (%.1f%%)\n", Oflag, def.types.omitted, 100.0*def.types.omitted/def.types.total);
       if (pflag)
-        fprintf(stderr, "\nWarning: option -O2 removed type definitions that may be used as xsd__anyType derivatives by type extension inheritance (enabled by default with option -p), use option -P to disable type derivation from xsd__anyType\n");
+        fprintf(stderr, "\nWarning: option -O%d removes type definitions that may be used as xsd__anyType derivatives by type extension inheritance (enabled by default with option -p), use option -P to disable type derivation from xsd__anyType\n", Oflag);
     }
     else
     {
-      fprintf(stderr, "\nOptimization (-O2): no unused schema components found and removed\n");
+      fprintf(stderr, "\nOptimization (-O%d): no unused schema components found\n", Oflag);
     }
   }
   if (outfile)
@@ -292,6 +295,9 @@ static void options(int argc, char **argv)
           case 'e':
             eflag = 1;
             break;
+          case 'F':
+            Fflag = 1;
+            break;
           case 'f':
             fflag = 1;
             break;
@@ -317,6 +323,9 @@ static void options(int argc, char **argv)
             else
               fprintf(stderr, "wsdl2h: Option -I requires a path argument\n");
             break;
+          case 'L':
+            Lflag = 1;
+            break;
           case 'l':
             fprintf(stderr, "%s", licensenotice);
             exit(0);
@@ -369,7 +378,7 @@ static void options(int argc, char **argv)
             else if (i < argc && argv[++i])
               Oflag = soap_strtol(argv[i], NULL, 10);
             else
-              fprintf(stderr, "wsdl2h: Option -O requires 1 or 2\n");
+              fprintf(stderr, "wsdl2h: Option -O requires an argument number\n");
             break;
           case 'p':
             pflag = 1;
@@ -502,43 +511,47 @@ static void options(int argc, char **argv)
             break;
           case '?':
           case 'h':
-            fprintf(stderr, "Usage: wsdl2h [-a] [-b] [-c|-c++|-c++11] [-D] [-d] [-e] [-f] [-g] [-h] [-I path] [-i] [-j] [-k] [-l] [-m] [-M] [-N name] [-n name] [-O1|-O2] [-P|-p] [-q name] [-R] [-r proxyhost[:port[:uid:pwd]]] [-r:userid:passwd] [-s] [-Sname] [-t typemapfile] [-U] [-u] [-V] [-v] [-w] [-W] [-x] [-y] [-z#] [-_] [-o outfile.h] infile.wsdl infile.xsd http://www... ...\n\n");
+            fprintf(stderr, "Usage: wsdl2h [-a] [-b] [-c|-c++|-c++11] [-D] [-d] [-e] [-F] [-f] [-g] [-h] [-I path] [-i] [-j] [-k] [-L] [-l] [-M] [-m] [-N name] [-n name] [-O1|-O2|-O3] [-P|-p] [-q name] [-R] [-r proxyhost[:port[:uid:pwd]]] [-r:uid:pwd] [-Sname] [-s] [-t typemapfile] [-U] [-u] [-V] [-v] [-w] [-W] [-x] [-y] [-z#] [-_] [-o outfile.h] infile.wsdl infile.xsd http://www... ...\n\n");
             fprintf(stderr, "\
 -a      generate indexed struct names for local elements with anonymous types\n\
 -b      bi-directional operations (duplex ops) added to serve one-way responses\n\
 -c      generate C source code\n\
 -c++    generate C++ source code (default)\n\
 -c++11  generate C++11 source code\n\
+-D      make attribute members with default/fixed values optional with pointers\n\
 -d      use DOM to populate xs:any, xs:anyType, and xs:anyAttribute\n\
--D      make attribute members with default values optional with pointers\n\
 -e      don't qualify enum names\n\
--f      generate flat C++ class hierarchy\n\
--g      generate global top-level element declarations\n\
--h      display help info\n\
--Ipath  use path to find files\n\
+-F      add transient members to structs to simulate struct-type derivation in C\n\
+-f      generate flat C++ class hierarchy by removing inheritance\n\
+-g      generate global top-level element and attribute declarations\n\
+-h      display help info and exit\n\
+-Ipath  use path to locate WSDL and XSD files\n\
 -i      don't import (advanced option)\n\
 -j      don't generate SOAP_ENV__Header and SOAP_ENV__Detail definitions\n\
 -k      don't generate SOAP_ENV__Header mustUnderstand qualifiers\n\
+-L      generate less documentation by removing generic @note comments\n\
 -l      display license information\n\
--m      use xsd.h module to import primitive types\n\
 -M      suppress error \"must understand element with wsdl:required='true'\"\n\
+-m      use xsd.h module to import primitive types\n\
 -Nname  use name for service prefixes to produce a service for each binding\n\
 -nname  use name as the base namespace prefix instead of 'ns'\n\
 -O1     optimize by omitting duplicate choice/sequence members\n\
 -O2     optimize -O1 and omit unused schema types (unreachable from roots)\n\
+-O3     optimize -O2 and omit unused schema root attributes\n\
+-O4     optimize -O3 and omit unused schema root elements (use only with WSDLs)\n\
 -ofile  output to file\n\
 -P      don't create polymorphic types inherited from xsd__anyType\n\
 -p      create polymorphic types inherited from base xsd__anyType\n\
 -qname  use name for the C++ namespace of all declarations\n\
 -R      generate REST operations for REST bindings specified in a WSDL\n\
 -rhost[:port[:uid:pwd]]\n\
-        connect via proxy host, port, and proxy credentials\n\
+        connect via proxy host, port, and proxy credentials uid and pwd\n\
 -r:uid:pwd\n\
-        connect with authentication credentials (digest auth requires SSL)\n\
--s      don't generate STL code (no std::string and no std::vector)\n\
+        connect with authentication credentials uid and pwd\n\
 -Sname  use name instead of 'soap' for the C++ class members with soap contexts\n\
+-s      don't generate STL code (no std::string and no std::vector)\n\
 -tfile  use type map file instead of the default file typemap.dat\n\
--U      allow UTF8-encoded Unicode C/C++ identifiers when mapping XML tag names\n\
+-U      allow UTF-8-encoded Unicode C/C++ identifiers when mapping XML tag names\n\
 -u      don't generate unions\n\
 -V      display the current version and exit\n\
 -v      verbose output\n\
@@ -553,7 +566,7 @@ static void options(int argc, char **argv)
 -z5     compatibility up to 2.8.15: don't include minor improvements\n\
 -z6     compatibility up to 2.8.17: don't include minor improvements\n\
 -z7     compatibility up to 2.8.59: don't generate std::vector of class of union\n\
--_      don't generate _USCORE (replace with UNICODE _x005f)\n\
+-_      don't generate _USCORE (replace with Unicode code point _x005f)\n\
 infile.wsdl infile.xsd http://www... list of input sources (if none reads stdin)\n\
 \n");
             exit(0);
